@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -85,7 +85,7 @@ g_mime_certificate_init (GMimeCertificate *cert, GMimeCertificateClass *klass)
 {
 	cert->pubkey_algo = GMIME_PUBKEY_ALGO_DEFAULT;
 	cert->digest_algo = GMIME_DIGEST_ALGO_DEFAULT;
-	cert->trust = GMIME_CERTIFICATE_TRUST_NONE;
+	cert->trust = GMIME_TRUST_UNKNOWN;
 	cert->issuer_serial = NULL;
 	cert->issuer_name = NULL;
 	cert->fingerprint = NULL;
@@ -122,19 +122,19 @@ g_mime_certificate_finalize (GObject *object)
 GMimeCertificate *
 g_mime_certificate_new (void)
 {
-	return g_object_newv (GMIME_TYPE_CERTIFICATE, 0, NULL);
+	return g_object_new (GMIME_TYPE_CERTIFICATE, NULL);
 }
 
 
 /**
  * g_mime_certificate_set_trust:
  * @cert: a #GMimeCertificate
- * @trust: a #GMimeCertificateTrust value
+ * @trust: a #GMimeTrust value
  *
  * Set the certificate trust.
  **/
 void
-g_mime_certificate_set_trust (GMimeCertificate *cert, GMimeCertificateTrust trust)
+g_mime_certificate_set_trust (GMimeCertificate *cert, GMimeTrust trust)
 {
 	g_return_if_fail (GMIME_IS_CERTIFICATE (cert));
 	
@@ -150,10 +150,10 @@ g_mime_certificate_set_trust (GMimeCertificate *cert, GMimeCertificateTrust trus
  *
  * Returns: the certificate trust.
  **/
-GMimeCertificateTrust
+GMimeTrust
 g_mime_certificate_get_trust (GMimeCertificate *cert)
 {
-	g_return_val_if_fail (GMIME_IS_CERTIFICATE (cert), GMIME_CERTIFICATE_TRUST_NONE);
+	g_return_val_if_fail (GMIME_IS_CERTIFICATE (cert), GMIME_TRUST_UNKNOWN);
 	
 	return cert->trust;
 }
@@ -550,13 +550,10 @@ static void
 g_mime_certificate_list_finalize (GObject *object)
 {
 	GMimeCertificateList *list = (GMimeCertificateList *) object;
-	GMimeCertificate *cert;
 	guint i;
 	
-	for (i = 0; i < list->array->len; i++) {
-		cert = (GMimeCertificate *) list->array->pdata[i];
-		g_object_unref (cert);
-	}
+	for (i = 0; i < list->array->len; i++)
+		g_object_unref (list->array->pdata[i]);
 	
 	g_ptr_array_free (list->array, TRUE);
 	
@@ -574,7 +571,7 @@ g_mime_certificate_list_finalize (GObject *object)
 GMimeCertificateList *
 g_mime_certificate_list_new (void)
 {
-	return g_object_newv (GMIME_TYPE_CERTIFICATE_LIST, 0, NULL);
+	return g_object_new (GMIME_TYPE_CERTIFICATE_LIST, NULL);
 }
 
 
@@ -599,7 +596,7 @@ g_mime_certificate_list_length (GMimeCertificateList *list)
  * g_mime_certificate_list_clear:
  * @list: a #GMimeCertificateList
  *
- * Clears the list of addresses.
+ * Clears the list of certificates.
  **/
 void
 g_mime_certificate_list_clear (GMimeCertificateList *list)
@@ -701,21 +698,19 @@ g_mime_certificate_list_remove (GMimeCertificateList *list, GMimeCertificate *ce
 	if ((index = g_mime_certificate_list_index_of (list, cert)) == -1)
 		return FALSE;
 	
-	g_mime_certificate_list_remove_at (list, index);
-	
-	return TRUE;
+	return g_mime_certificate_list_remove_at (list, index);
 }
 
 
 /**
  * g_mime_certificate_list_remove_at:
  * @list: a #GMimeCertificateList
- * @index: index to remove
+ * @index: index of the certificate to remove
  *
  * Removes a #GMimeCertificate from the #GMimeCertificateList at the specified
  * index.
  *
- * Returns: %TRUE if an #GMimeCertificate was removed or %FALSE otherwise.
+ * Returns: %TRUE if a #GMimeCertificate was removed or %FALSE otherwise.
  **/
 gboolean
 g_mime_certificate_list_remove_at (GMimeCertificateList *list, int index)
