@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2013 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -65,22 +65,28 @@ typedef struct _GMimeCertificateListClass GMimeCertificateListClass;
  * @GMIME_DIGEST_ALGO_SHA512: The SHA-512 hash algorithm.
  * @GMIME_DIGEST_ALGO_SHA224: The SHA-224 hash algorithm.
  * @GMIME_DIGEST_ALGO_MD4: The MD4 hash algorithm.
+ * @GMIME_DIGEST_ALGO_CRC32: The CRC32 hash algorithm.
+ * @GMIME_DIGEST_ALGO_CRC32_RFC1510: The rfc1510 CRC32 hash algorithm.
+ * @GMIME_DIGEST_ALGO_CRC32_RFC2440: The rfc2440 CRC32 hash algorithm.
  *
  * A hash algorithm.
  **/
 typedef enum {
-	GMIME_DIGEST_ALGO_DEFAULT     = 0,
-	GMIME_DIGEST_ALGO_MD5         = 1,
-	GMIME_DIGEST_ALGO_SHA1        = 2,
-	GMIME_DIGEST_ALGO_RIPEMD160   = 3,
-	GMIME_DIGEST_ALGO_MD2         = 5,
-	GMIME_DIGEST_ALGO_TIGER192    = 6,
-	GMIME_DIGEST_ALGO_HAVAL5160   = 7,
-	GMIME_DIGEST_ALGO_SHA256      = 8,
-	GMIME_DIGEST_ALGO_SHA384      = 9,
-	GMIME_DIGEST_ALGO_SHA512      = 10,
-	GMIME_DIGEST_ALGO_SHA224      = 11,
-	GMIME_DIGEST_ALGO_MD4         = 301
+	GMIME_DIGEST_ALGO_DEFAULT       = 0,
+	GMIME_DIGEST_ALGO_MD5           = 1,
+	GMIME_DIGEST_ALGO_SHA1          = 2,
+	GMIME_DIGEST_ALGO_RIPEMD160     = 3,
+	GMIME_DIGEST_ALGO_MD2           = 5,
+	GMIME_DIGEST_ALGO_TIGER192      = 6,
+	GMIME_DIGEST_ALGO_HAVAL5160     = 7,
+	GMIME_DIGEST_ALGO_SHA256        = 8,
+	GMIME_DIGEST_ALGO_SHA384        = 9,
+	GMIME_DIGEST_ALGO_SHA512        = 10,
+	GMIME_DIGEST_ALGO_SHA224        = 11,
+	GMIME_DIGEST_ALGO_MD4           = 301,
+	GMIME_DIGEST_ALGO_CRC32         = 302,
+	GMIME_DIGEST_ALGO_CRC32_RFC1510 = 303,
+	GMIME_DIGEST_ALGO_CRC32_RFC2440 = 304
 } GMimeDigestAlgo;
 
 /**
@@ -91,7 +97,11 @@ typedef enum {
  * @GMIME_PUBKEY_ALGO_RSA_S: A signature-only RSA algorithm.
  * @GMIME_PUBKEY_ALGO_ELG_E: An encryption-only ElGamal algorithm.
  * @GMIME_PUBKEY_ALGO_DSA: The DSA algorithm.
+ * @GMIME_PUBKEY_ALGO_ECC: The Eliptic Curve algorithm.
  * @GMIME_PUBKEY_ALGO_ELG: The ElGamal algorithm.
+ * @GMIME_PUBKEY_ALGO_ECDSA: The Eliptic Curve + DSA algorithm.
+ * @GMIME_PUBKEY_ALGO_ECDH: The Eliptic Curve + Diffie Helman algorithm.
+ * @GMIME_PUBKEY_ALGO_EDDSA: The Eliptic Curve + DSA algorithm.
  *
  * A public-key algorithm.
  **/
@@ -102,28 +112,32 @@ typedef enum {
 	GMIME_PUBKEY_ALGO_RSA_S    = 3,
 	GMIME_PUBKEY_ALGO_ELG_E    = 16,
 	GMIME_PUBKEY_ALGO_DSA      = 17,
-	GMIME_PUBKEY_ALGO_ELG      = 20
+	GMIME_PUBKEY_ALGO_ECC      = 18,
+	GMIME_PUBKEY_ALGO_ELG      = 20,
+	GMIME_PUBKEY_ALGO_ECDSA    = 301,
+	GMIME_PUBKEY_ALGO_ECDH     = 302,
+	GMIME_PUBKEY_ALGO_EDDSA    = 303
 } GMimePubKeyAlgo;
 
 /**
- * GMimeCertificateTrust:
- * @GMIME_CERTIFICATE_TRUST_NONE: No trust assigned.
- * @GMIME_CERTIFICATE_TRUST_NEVER: Never trust this certificate.
- * @GMIME_CERTIFICATE_TRUST_UNDEFINED: Undefined trust for this certificate.
- * @GMIME_CERTIFICATE_TRUST_MARGINAL: Trust this certificate maginally.
- * @GMIME_CERTIFICATE_TRUST_FULLY: Trust this certificate fully.
- * @GMIME_CERTIFICATE_TRUST_ULTIMATE: Trust this certificate ultimately.
+ * GMimeTrust:
+ * @GMIME_TRUST_UNKNOWN: The certificate or key is of unknown validity.
+ * @GMIME_TRUST_UNDEFINED: The validity of the certificate or key is undefined.
+ * @GMIME_TRUST_NEVER: The certificate or key should never be treated as valid.
+ * @GMIME_TRUST_MARGINAL: The certificate or key is marginally valid.
+ * @GMIME_TRUST_FULL: The certificate or key is fully valid.
+ * @GMIME_TRUST_ULTIMATE: The certificate or key is ultimately valid.
  *
- * The trust value of a certificate.
+ * The trust level of a certificate or key.
  **/
 typedef enum {
-	GMIME_CERTIFICATE_TRUST_NONE,
-	GMIME_CERTIFICATE_TRUST_NEVER,
-	GMIME_CERTIFICATE_TRUST_UNDEFINED,
-	GMIME_CERTIFICATE_TRUST_MARGINAL,
-	GMIME_CERTIFICATE_TRUST_FULLY,
-	GMIME_CERTIFICATE_TRUST_ULTIMATE
-} GMimeCertificateTrust;
+	GMIME_TRUST_UNKNOWN   = 0,
+	GMIME_TRUST_UNDEFINED = 1,
+	GMIME_TRUST_NEVER     = 2,
+	GMIME_TRUST_MARGINAL  = 3,
+	GMIME_TRUST_FULL      = 4,
+	GMIME_TRUST_ULTIMATE  = 5
+} GMimeTrust;
 
 /**
  * GMimeCertificate:
@@ -147,7 +161,7 @@ struct _GMimeCertificate {
 	
 	GMimePubKeyAlgo pubkey_algo;
 	GMimeDigestAlgo digest_algo;
-	GMimeCertificateTrust trust;
+	GMimeTrust trust;
 	char *issuer_serial;
 	char *issuer_name;
 	char *fingerprint;
@@ -168,8 +182,8 @@ GType g_mime_certificate_get_type (void);
 
 GMimeCertificate *g_mime_certificate_new (void);
 
-void g_mime_certificate_set_trust (GMimeCertificate *cert, GMimeCertificateTrust trust);
-GMimeCertificateTrust g_mime_certificate_get_trust (GMimeCertificate *cert);
+void g_mime_certificate_set_trust (GMimeCertificate *cert, GMimeTrust trust);
+GMimeTrust g_mime_certificate_get_trust (GMimeCertificate *cert);
 
 void g_mime_certificate_set_pubkey_algo (GMimeCertificate *cert, GMimePubKeyAlgo algo);
 GMimePubKeyAlgo g_mime_certificate_get_pubkey_algo (GMimeCertificate *cert);
