@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -57,6 +57,8 @@ static void g_mime_content_type_class_init (GMimeContentTypeClass *klass);
 static void g_mime_content_type_init (GMimeContentType *content_type, GMimeContentTypeClass *klass);
 static void g_mime_content_type_finalize (GObject *object);
 
+static void param_list_changed (GMimeParamList *list, gpointer args, GMimeContentType *content_type);
+
 
 static GObjectClass *parent_class = NULL;
 
@@ -99,11 +101,12 @@ g_mime_content_type_class_init (GMimeContentTypeClass *klass)
 static void
 g_mime_content_type_init (GMimeContentType *content_type, GMimeContentTypeClass *klass)
 {
-	content_type->param_hash = g_hash_table_new (g_mime_strcase_hash, g_mime_strcase_equal);
-	content_type->priv = g_mime_event_new ((GObject *) content_type);
-	content_type->params = NULL;
+	content_type->changed = g_mime_event_new ((GObject *) content_type);
+	content_type->params = g_mime_param_list_new ();
 	content_type->subtype = NULL;
 	content_type->type = NULL;
+	
+	g_mime_event_add (content_type->params->changed, (GMimeEventCallback) param_list_changed, content_type);
 }
 
 static void
@@ -111,9 +114,8 @@ g_mime_content_type_finalize (GObject *object)
 {
 	GMimeContentType *content_type = (GMimeContentType *) object;
 	
-	g_hash_table_destroy (content_type->param_hash);
-	g_mime_param_destroy (content_type->params);
-	g_mime_event_destroy (content_type->priv);
+	g_mime_event_free (content_type->changed);
+	g_object_unref (content_type->params);
 	g_free (content_type->subtype);
 	g_free (content_type->type);
 	
@@ -121,10 +123,17 @@ g_mime_content_type_finalize (GObject *object)
 }
 
 
+static void
+param_list_changed (GMimeParamList *list, gpointer args, GMimeContentType *content_type)
+{
+	g_mime_event_emit (content_type->changed, NULL);
+}
+
+
 /**
  * g_mime_content_type_new:
- * @type: MIME type (or %NULL for "text")
- * @subtype: MIME subtype (or %NULL for "plain")
+ * @type: the MIME type or %NULL for the default value
+ * @subtype: the MIME subtype or %NULL for the default value
  *
  * Creates a Content-Type object with type @type and subtype @subtype.
  *
@@ -133,52 +142,54 @@ g_mime_content_type_finalize (GObject *object)
 GMimeContentType *
 g_mime_content_type_new (const char *type, const char *subtype)
 {
-	GMimeContentType *mime_type;
+	GMimeContentType *content_type;
 	
-	mime_type = g_object_newv (GMIME_TYPE_CONTENT_TYPE, 0, NULL);
+	content_type = g_object_new (GMIME_TYPE_CONTENT_TYPE, NULL);
 	
 	if (type && *type && subtype && *subtype) {
-		mime_type->type = g_strdup (type);
-		mime_type->subtype = g_strdup (subtype);
+		content_type->type = g_strdup (type);
+		content_type->subtype = g_strdup (subtype);
 	} else {
 		if (type && *type) {
-			mime_type->type = g_strdup (type);
+			content_type->type = g_strdup (type);
 			if (!g_ascii_strcasecmp (type, "text")) {
-				mime_type->subtype = g_strdup ("plain");
+				content_type->subtype = g_strdup ("plain");
 			} else if (!g_ascii_strcasecmp (type, "multipart")) {
-				mime_type->subtype = g_strdup ("mixed");
+				content_type->subtype = g_strdup ("mixed");
 			} else {
-				g_free (mime_type->type);
-				mime_type->type = g_strdup ("application");
-				mime_type->subtype = g_strdup ("octet-stream");
+				g_free (content_type->type);
+				content_type->type = g_strdup ("application");
+				content_type->subtype = g_strdup ("octet-stream");
 			}
 		} else {
-			mime_type->type = g_strdup ("application");
-			mime_type->subtype = g_strdup ("octet-stream");
+			content_type->type = g_strdup ("application");
+			content_type->subtype = g_strdup ("octet-stream");
 		}
 		
 		w(g_warning ("Invalid or incomplete type: %s%s%s: defaulting to %s/%s",
 			     type ? type : "", subtype ? "/" : "", subtype ? subtype : "",
-			     mime_type->type, mime_type->subtype));
+			     content_type->type, content_type->subtype));
 	}
 	
-	return mime_type;
+	return content_type;
 }
 
 
 /**
- * g_mime_content_type_new_from_string:
+ * g_mime_content_type_parse:
+ * @options: (nullable): a #GMimeParserOptions or %NULL
  * @str: input string containing a content-type (and params)
  *
- * Constructs a new Content-Type object based on the input string.
+ * Parses the input string into a #GMimeContentType object.
  *
- * Returns: a new #GMimeContentType object based on the input string.
+ * Returns: (transfer full): a new #GMimeContentType object.
  **/
 GMimeContentType *
-g_mime_content_type_new_from_string (const char *str)
+g_mime_content_type_parse (GMimeParserOptions *options, const char *str)
 {
-	GMimeContentType *mime_type;
+	GMimeContentType *content_type;
 	const char *inptr = str;
+	GMimeParamList *params;
 	char *type, *subtype;
 	
 	g_return_val_if_fail (str != NULL, NULL);
@@ -186,82 +197,116 @@ g_mime_content_type_new_from_string (const char *str)
 	if (!g_mime_parse_content_type (&inptr, &type, &subtype))
 		return g_mime_content_type_new ("application", "octet-stream");
 	
-	mime_type = g_object_newv (GMIME_TYPE_CONTENT_TYPE, 0, NULL);
-	mime_type->subtype = subtype;
-	mime_type->type = type;
+	content_type = g_object_new (GMIME_TYPE_CONTENT_TYPE, NULL);
+	content_type->subtype = subtype;
+	content_type->type = type;
 	
 	/* skip past any remaining junk that shouldn't be here... */
-	decode_lwsp (&inptr);
+	skip_cfws (&inptr);
 	while (*inptr && *inptr != ';')
 		inptr++;
 	
-	if (*inptr++ == ';' && *inptr) {
-		GMimeParam *param;
-		
-		param = mime_type->params = g_mime_param_new_from_string (inptr);
-		while (param != NULL) {
-			g_hash_table_insert (mime_type->param_hash, param->name, param);
-			param = param->next;
-		}
+	if (*inptr++ == ';' && *inptr && (params = g_mime_param_list_parse (options, inptr))) {
+		g_mime_event_add (params->changed, (GMimeEventCallback) param_list_changed, content_type);
+		g_object_unref (content_type->params);
+		content_type->params = params;
 	}
 	
+	return content_type;
+}
+
+
+/**
+ * g_mime_content_type_get_mime_type:
+ * @content_type: a #GMimeContentType
+ *
+ * Allocates a string buffer containing the type and subtype defined
+ * by the @content_type.
+ *
+ * Returns: an allocated string containing the type and subtype of the
+ * content-type in the format: type/subtype.
+ **/
+char *
+g_mime_content_type_get_mime_type (GMimeContentType *content_type)
+{
+	char *mime_type;
+	
+	g_return_val_if_fail (GMIME_IS_CONTENT_TYPE (content_type), NULL);
+	
+	/* type and subtype should never be NULL, but check anyway */
+	mime_type = g_strdup_printf ("%s/%s", content_type->type ? content_type->type : "text",
+				     content_type->subtype ? content_type->subtype : "plain");
+	
 	return mime_type;
 }
 
 
 /**
- * g_mime_content_type_to_string:
- * @mime_type: a #GMimeContentType object
+ * g_mime_content_type_encode:
+ * @content_type: a #GMimeContentType
+ * @options: (nullable): a #GMimeFormatOptions or %NULL
  *
- * Allocates a string buffer containing the type and subtype defined
- * by the @mime_type.
+ * Encodes the Content-Disposition header.
  *
- * Returns: an allocated string containing the type and subtype of the
- * content-type in the format: type/subtype.
+ * Returns: a new string containing the encoded header value.
  **/
 char *
-g_mime_content_type_to_string (GMimeContentType *mime_type)
+g_mime_content_type_encode (GMimeContentType *content_type, GMimeFormatOptions *options)
 {
-	char *string;
+	char *raw_value;
+	GString *str;
+	guint len, n;
 	
-	g_return_val_if_fail (GMIME_IS_CONTENT_TYPE (mime_type), NULL);
+	g_return_val_if_fail (GMIME_IS_CONTENT_TYPE (content_type), NULL);
 	
-	/* type and subtype should never be NULL, but check anyway */
-	string = g_strdup_printf ("%s/%s", mime_type->type ? mime_type->type : "text",
-				  mime_type->subtype ? mime_type->subtype : "plain");
+	/* we need to have this so wrapping is correct */
+	str = g_string_new ("Content-Type:");
+	n = str->len;
 	
-	return string;
+	g_string_append_c (str, ' ');
+	g_string_append (str, content_type->type ? content_type->type : "text");
+	g_string_append_c (str, '/');
+	g_string_append (str, content_type->subtype ? content_type->subtype : "plain");
+	
+	g_mime_param_list_encode (content_type->params, options, TRUE, str);
+	len = str->len - n;
+	
+	raw_value = g_string_free (str, FALSE);
+	
+	memmove (raw_value, raw_value + n, len + 1);
+	
+	return raw_value;
 }
 
 
 /**
  * g_mime_content_type_is_type:
- * @mime_type: a #GMimeContentType object
+ * @content_type: a #GMimeContentType
  * @type: MIME type to compare against
  * @subtype: MIME subtype to compare against
  *
  * Compares the given type and subtype with that of the given mime
  * type object.
  *
- * Returns: TRUE if the MIME types match or FALSE otherwise. You may
+ * Returns: %TRUE if the MIME types match or %FALSE otherwise. You may
  * use "*" in place of @type and/or @subtype as a wilcard.
  **/
 gboolean
-g_mime_content_type_is_type (GMimeContentType *mime_type, const char *type, const char *subtype)
+g_mime_content_type_is_type (GMimeContentType *content_type, const char *type, const char *subtype)
 {
-	g_return_val_if_fail (GMIME_IS_CONTENT_TYPE (mime_type), FALSE);
-	g_return_val_if_fail (mime_type->type != NULL, FALSE);
-	g_return_val_if_fail (mime_type->subtype != NULL, FALSE);
+	g_return_val_if_fail (GMIME_IS_CONTENT_TYPE (content_type), FALSE);
+	g_return_val_if_fail (content_type->type != NULL, FALSE);
+	g_return_val_if_fail (content_type->subtype != NULL, FALSE);
 	g_return_val_if_fail (type != NULL, FALSE);
 	g_return_val_if_fail (subtype != NULL, FALSE);
 	
-	if (!strcmp (type, "*") || !g_ascii_strcasecmp (mime_type->type, type)) {
+	if (!strcmp (type, "*") || !g_ascii_strcasecmp (content_type->type, type)) {
 		if (!strcmp (subtype, "*")) {
 			/* special case */
 			return TRUE;
 		}
 		
-		if (!g_ascii_strcasecmp (mime_type->subtype, subtype))
+		if (!g_ascii_strcasecmp (content_type->subtype, subtype))
 			return TRUE;
 	}
 	
@@ -271,130 +316,104 @@ g_mime_content_type_is_type (GMimeContentType *mime_type, const char *type, cons
 
 /**
  * g_mime_content_type_set_media_type:
- * @mime_type: a #GMimeContentType object
+ * @content_type: a #GMimeContentType
  * @type: media type
  *
  * Sets the Content-Type's media type.
  **/
 void
-g_mime_content_type_set_media_type (GMimeContentType *mime_type, const char *type)
+g_mime_content_type_set_media_type (GMimeContentType *content_type, const char *type)
 {
 	char *buf;
 	
-	g_return_if_fail (GMIME_IS_CONTENT_TYPE (mime_type));
+	g_return_if_fail (GMIME_IS_CONTENT_TYPE (content_type));
 	g_return_if_fail (type != NULL);
 	
 	buf = g_strdup (type);
-	g_free (mime_type->type);
-	mime_type->type = buf;
+	g_free (content_type->type);
+	content_type->type = buf;
 	
-	g_mime_event_emit (mime_type->priv, NULL);
+	g_mime_event_emit (content_type->changed, NULL);
 }
 
 
 /**
  * g_mime_content_type_get_media_type:
- * @mime_type: a #GMimeContentType object
+ * @content_type: a #GMimeContentType
  *
  * Gets the Content-Type's media type.
  *
  * Returns: the Content-Type's media type.
  **/
 const char *
-g_mime_content_type_get_media_type (GMimeContentType *mime_type)
+g_mime_content_type_get_media_type (GMimeContentType *content_type)
 {
-	g_return_val_if_fail (GMIME_IS_CONTENT_TYPE (mime_type), NULL);
+	g_return_val_if_fail (GMIME_IS_CONTENT_TYPE (content_type), NULL);
 	
-	return mime_type->type;
+	return content_type->type;
 }
 
 
 /**
  * g_mime_content_type_set_media_subtype:
- * @mime_type: a #GMimeContentType object
+ * @content_type: a #GMimeContentType
  * @subtype: media subtype
  *
  * Sets the Content-Type's media subtype.
  **/
 void
-g_mime_content_type_set_media_subtype (GMimeContentType *mime_type, const char *subtype)
+g_mime_content_type_set_media_subtype (GMimeContentType *content_type, const char *subtype)
 {
 	char *buf;
 	
-	g_return_if_fail (GMIME_IS_CONTENT_TYPE (mime_type));
+	g_return_if_fail (GMIME_IS_CONTENT_TYPE (content_type));
 	g_return_if_fail (subtype != NULL);
 	
 	buf = g_strdup (subtype);
-	g_free (mime_type->subtype);
-	mime_type->subtype = buf;
+	g_free (content_type->subtype);
+	content_type->subtype = buf;
 	
-	g_mime_event_emit (mime_type->priv, NULL);
+	g_mime_event_emit (content_type->changed, NULL);
 }
 
 
 /**
  * g_mime_content_type_get_media_subtype:
- * @mime_type: a #GMimeContentType object
+ * @content_type: a #GMimeContentType
  *
  * Gets the Content-Type's media sub-type.
  *
  * Returns: the Content-Type's media sub-type.
  **/
 const char *
-g_mime_content_type_get_media_subtype (GMimeContentType *mime_type)
+g_mime_content_type_get_media_subtype (GMimeContentType *content_type)
 {
-	g_return_val_if_fail (GMIME_IS_CONTENT_TYPE (mime_type), NULL);
+	g_return_val_if_fail (GMIME_IS_CONTENT_TYPE (content_type), NULL);
 	
-	return mime_type->subtype;
+	return content_type->subtype;
 }
 
 
 /**
- * g_mime_content_type_set_params:
- * @mime_type: a #GMimeContentType object
- * @params: a list of #GMimeParam objects
- *
- * Sets the Content-Type's parameter list.
- **/
-void
-g_mime_content_type_set_params (GMimeContentType *mime_type, GMimeParam *params)
-{
-	g_return_if_fail (GMIME_IS_CONTENT_TYPE (mime_type));
-	
-	/* clear the current list/hash */
-	g_hash_table_remove_all (mime_type->param_hash);
-	g_mime_param_destroy (mime_type->params);
-	mime_type->params = params;
-	
-	while (params != NULL) {
-		g_hash_table_insert (mime_type->param_hash, params->name, params);
-		params = params->next;
-	}
-	
-	g_mime_event_emit (mime_type->priv, NULL);
-}
-
-
-/**
- * g_mime_content_type_get_params:
- * @mime_type: a #GMimeContentType object
+ * g_mime_content_type_get_parameters:
+ * @content_type: a #GMimeContentType
  *
  * Gets the Content-Type's parameter list.
  *
- * Returns: the Content-Type's parameter list.
+ * Returns: (transfer none): the Content-Type's parameter list.
  **/
-const GMimeParam *
-g_mime_content_type_get_params (GMimeContentType *mime_type)
+GMimeParamList *
+g_mime_content_type_get_parameters (GMimeContentType *content_type)
 {
-	g_return_val_if_fail (GMIME_IS_CONTENT_TYPE (mime_type), NULL);
+	g_return_val_if_fail (GMIME_IS_CONTENT_TYPE (content_type), NULL);
 	
-	return mime_type->params;
+	return content_type->params;
 }
 
 
 /**
  * g_mime_content_type_set_parameter:
- * @mime_type: MIME Content-Type
+ * @content_type: a #GMimeContentType
  * @name: parameter name (aka attribute)
  * @value: parameter value
  *
@@ -404,30 +423,17 @@ g_mime_content_type_get_params (GMimeContentType *mime_type)
  * UTF-8.
  **/
 void
-g_mime_content_type_set_parameter (GMimeContentType *mime_type, const char *name, const char *value)
+g_mime_content_type_set_parameter (GMimeContentType *content_type, const char *name, const char *value)
 {
-	GMimeParam *param = NULL;
+	g_return_if_fail (GMIME_IS_CONTENT_TYPE (content_type));
 	
-	g_return_if_fail (GMIME_IS_CONTENT_TYPE (mime_type));
-	g_return_if_fail (name != NULL);
-	g_return_if_fail (value != NULL);
-	
-	if ((param = g_hash_table_lookup (mime_type->param_hash, name))) {
-		g_free (param->value);
-		param->value = g_strdup (value);
-	} else {
-		param = g_mime_param_new (name, value);
-		mime_type->params = g_mime_param_append_param (mime_type->params, param);
-		g_hash_table_insert (mime_type->param_hash, param->name, param);
-	}
-	
-	g_mime_event_emit (mime_type->priv, NULL);
+	g_mime_param_list_set_parameter (content_type->params, name, value);
 }
 
 
 /**
  * g_mime_content_type_get_parameter:
- * @mime_type: a #GMimeContentType object
+ * @content_type: a #GMimeContentType
  * @name: parameter name (aka attribute)
  *
  * Gets the parameter value specified by @name if it's available.
@@ -437,14 +443,13 @@ g_mime_content_type_set_parameter (GMimeContentType *mime_type, const char *name
  * will be in UTF-8.
  **/
 const char *
-g_mime_content_type_get_parameter (GMimeContentType *mime_type, const char *name)
+g_mime_content_type_get_parameter (GMimeContentType *content_type, const char *name)
 {
 	GMimeParam *param;
 	
-	g_return_val_if_fail (GMIME_IS_CONTENT_TYPE (mime_type), NULL);
-	g_return_val_if_fail (name != NULL, NULL);
+	g_return_val_if_fail (GMIME_IS_CONTENT_TYPE (content_type), NULL);
 	
-	if (!(param = g_hash_table_lookup (mime_type->param_hash, name)))
+	if (!(param = g_mime_param_list_get_parameter (content_type->params, name)))
 		return NULL;
 	
 	return param->value;
