@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -26,9 +26,11 @@
 #include <ctype.h>
 #include <string.h>
 
+#include "gmime-table-private.h"
 #include "gmime-common.h"
 #include "gmime-object.h"
 #include "gmime-stream-mem.h"
+#include "gmime-internal.h"
 #include "gmime-events.h"
 #include "gmime-utils.h"
 
@@ -43,7 +45,6 @@
  * parts are derived.
  **/
 
-
 struct _type_bucket {
 	char *type;
 	GType object_type;
@@ -62,19 +63,17 @@ static void g_mime_object_class_init (GMimeObjectClass *klass);
 static void g_mime_object_init (GMimeObject *object, GMimeObjectClass *klass);
 static void g_mime_object_finalize (GObject *object);
 
-static void object_prepend_header (GMimeObject *object, const char *name, const char *value);
-static void object_append_header (GMimeObject *object, const char *name, const char *value);
-static void object_set_header (GMimeObject *object, const char *name, const char *value);
-static const char *object_get_header (GMimeObject *object, const char *name);
-static gboolean object_remove_header (GMimeObject *object, const char *name);
+static void object_header_added    (GMimeObject *object, GMimeHeader *header);
+static void object_header_changed  (GMimeObject *object, GMimeHeader *header);
+static void object_header_removed  (GMimeObject *object, GMimeHeader *header);
+static void object_headers_cleared (GMimeObject *object);
+
 static void object_set_content_type (GMimeObject *object, GMimeContentType *content_type);
-static char *object_get_headers (GMimeObject *object);
-static ssize_t object_write_to_stream (GMimeObject *object, GMimeStream *stream);
+static char *object_get_headers (GMimeObject *object, GMimeFormatOptions *options);
+static ssize_t object_write_to_stream (GMimeObject *object, GMimeFormatOptions *options, gboolean content_only, GMimeStream *stream);
 static void object_encode (GMimeObject *object, GMimeEncodingConstraint constraint);
 
-static ssize_t write_content_type (GMimeStream *stream, const char *name, const char *value);
-static ssize_t write_disposition (GMimeStream *stream, const char *name, const char *value);
-
+static void header_list_changed (GMimeHeaderList *headers, GMimeHeaderListChangedEventArgs *args, GMimeObject *object);
 static void content_type_changed (GMimeContentType *content_type, gpointer args, GMimeObject *object);
 static void content_disposition_changed (GMimeContentDisposition *disposition, gpointer args, GMimeObject *object);
 
@@ -119,11 +118,10 @@ g_mime_object_class_init (GMimeObjectClass *klass)
 	
 	object_class->finalize = g_mime_object_finalize;
 	
-	klass->prepend_header = object_prepend_header;
-	klass->append_header = object_append_header;
-	klass->remove_header = object_remove_header;
-	klass->set_header = object_set_header;
-	klass->get_header = object_get_header;
+	klass->header_added = object_header_added;
+	klass->header_changed = object_header_changed;
+	klass->header_removed = object_header_removed;
+	klass->headers_cleared = object_headers_cleared;
 	klass->set_content_type = object_set_content_type;
 	klass->get_headers = object_get_headers;
 	klass->write_to_stream = object_write_to_stream;
@@ -133,123 +131,219 @@ g_mime_object_class_init (GMimeObjectClass *klass)
 static void
 g_mime_object_init (GMimeObject *object, GMimeObjectClass *klass)
 {
-	object->headers = g_mime_header_list_new ();
+	GMimeHeaderList *headers;
+	
+	headers = g_mime_header_list_new (g_mime_parser_options_get_default ());
+	g_mime_event_add (headers->changed, (GMimeEventCallback) header_list_changed, object);
+	object->headers = headers;
+	
+	object->ensure_newline = FALSE;
 	object->content_type = NULL;
 	object->disposition = NULL;
 	object->content_id = NULL;
-	
-	g_mime_header_list_register_writer (object->headers, "Content-Type", write_content_type);
-	g_mime_header_list_register_writer (object->headers, "Content-Disposition", write_disposition);
 }
 
+
 static void
 g_mime_object_finalize (GObject *object)
 {
 	GMimeObject *mime = (GMimeObject *) object;
+	GMimeEvent *event;
 	
 	if (mime->content_type) {
-		g_mime_event_remove (mime->content_type->priv, (GMimeEventCallback) content_type_changed, object);
+		event = mime->content_type->changed;
+		g_mime_event_remove (event, (GMimeEventCallback) content_type_changed, object);
 		g_object_unref (mime->content_type);
 	}
 	
 	if (mime->disposition) {
-		g_mime_event_remove (mime->disposition->priv, (GMimeEventCallback) content_disposition_changed, object);
+		event = mime->disposition->changed;
+		g_mime_event_remove (event, (GMimeEventCallback) content_disposition_changed, object);
 		g_object_unref (mime->disposition);
 	}
 	
-	if (mime->headers)
-		g_mime_header_list_destroy (mime->headers);
+	if (mime->headers) {
+		event = mime->headers->changed;
+		g_mime_event_remove (event, (GMimeEventCallback) header_list_changed, object);
+		g_object_unref (mime->headers);
+	}
 	
 	g_free (mime->content_id);
 	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+enum {
+	HEADER_CONTENT_DISPOSITION,
+	HEADER_CONTENT_TYPE,
+	HEADER_CONTENT_ID,
+	HEADER_UNKNOWN,
+};
 
-static ssize_t
-write_content_type (GMimeStream *stream, const char *name, const char *value)
+static char *content_headers[] = {
+	"Content-Disposition",
+	"Content-Type",
+	"Content-Id",
+};
+
+static void
+object_header_added (GMimeObject *object, GMimeHeader *header)
 {
+	object_header_changed (object, header);
+}
+
+static void
+object_header_changed (GMimeObject *object, GMimeHeader *header)
+{
+	GMimeParserOptions *options = _g_mime_header_list_get_options (object->headers);
+	GMimeContentDisposition *disposition;
 	GMimeContentType *content_type;
-	ssize_t nwritten;
-	GString *out;
-	char *val;
+	const char *name, *value;
+	guint i;
 	
-	out = g_string_new ("");
-	g_string_printf (out, "%s: ", name);
+	name = g_mime_header_get_name (header);
 	
-	content_type = g_mime_content_type_new_from_string (value);
+	if (g_ascii_strncasecmp (name, "Content-", 8) != 0)
+		return;
 	
-	val = g_mime_content_type_to_string (content_type);
-	g_string_append (out, val);
-	g_free (val);
+	for (i = 0; i < G_N_ELEMENTS (content_headers); i++) {
+		if (!g_ascii_strcasecmp (content_headers[i] + 8, name + 8))
+			break;
+	}
 	
-	g_mime_param_write_to_string (content_type->params, TRUE, out);
-	g_object_unref (content_type);
+	switch (i) {
+	case HEADER_CONTENT_DISPOSITION:
+		value = g_mime_header_get_value (header);
+		disposition = g_mime_content_disposition_parse (options, value);
+		_g_mime_object_set_content_disposition (object, disposition);
+		g_object_unref (disposition);
+		break;
+	case HEADER_CONTENT_TYPE:
+		value = g_mime_header_get_value (header);
+		content_type = g_mime_content_type_parse (options, value);
+		_g_mime_object_set_content_type (object, content_type);
+		g_object_unref (content_type);
+		break;
+	case HEADER_CONTENT_ID:
+		value = g_mime_header_get_value (header);
+		g_free (object->content_id);
+		object->content_id = g_mime_utils_decode_message_id (value);
+		break;
+	}
+}
+
+static void
+object_header_removed (GMimeObject *object, GMimeHeader *header)
+{
+	GMimeEvent *event;
+	const char *name;
+	guint i;
 	
-	nwritten = g_mime_stream_write (stream, out->str, out->len);
-	g_string_free (out, TRUE);
+	name = g_mime_header_get_name (header);
 	
-	return nwritten;
+	if (g_ascii_strncasecmp (name, "Content-", 8) != 0)
+		return;
+	
+	for (i = 0; i < G_N_ELEMENTS (content_headers); i++) {
+		if (!g_ascii_strcasecmp (content_headers[i] + 8, name + 8))
+			break;
+	}
+	
+	switch (i) {
+	case HEADER_CONTENT_DISPOSITION:
+		if (object->disposition) {
+			event = object->disposition->changed;
+			g_mime_event_remove (event, (GMimeEventCallback) content_disposition_changed, object);
+			g_object_unref (object->disposition);
+			object->disposition = NULL;
+		}
+		break;
+	case HEADER_CONTENT_TYPE:
+		/* never allow the removal of the Content-Type header */
+		break;
+	case HEADER_CONTENT_ID:
+		g_free (object->content_id);
+		object->content_id = NULL;
+		break;
+	}
+}
+
+static void
+object_headers_cleared (GMimeObject *object)
+{
+	GMimeEvent *event;
+	
+	if (object->disposition) {
+		event = object->disposition->changed;
+		g_mime_event_remove (event, (GMimeEventCallback) content_disposition_changed, object);
+		g_object_unref (object->disposition);
+		object->disposition = NULL;
+	}
+	
+	g_free (object->content_id);
+	object->content_id = NULL;
+}
+
+static void
+header_list_changed (GMimeHeaderList *headers, GMimeHeaderListChangedEventArgs *args, GMimeObject *object)
+{
+	switch (args->action) {
+	case GMIME_HEADER_LIST_CHANGED_ACTION_ADDED:
+		GMIME_OBJECT_GET_CLASS (object)->header_added (object, args->header);
+		break;
+	case GMIME_HEADER_LIST_CHANGED_ACTION_CHANGED:
+		GMIME_OBJECT_GET_CLASS (object)->header_changed (object, args->header);
+		break;
+	case GMIME_HEADER_LIST_CHANGED_ACTION_REMOVED:
+		GMIME_OBJECT_GET_CLASS (object)->header_removed (object, args->header);
+		break;
+	case GMIME_HEADER_LIST_CHANGED_ACTION_CLEARED:
+		GMIME_OBJECT_GET_CLASS (object)->headers_cleared (object);
+		break;
+	}
+}
+
+void
+_g_mime_object_block_header_list_changed (GMimeObject *object)
+{
+	g_mime_event_block (object->headers->changed, (GMimeEventCallback) header_list_changed, object);
+}
+
+void
+_g_mime_object_unblock_header_list_changed (GMimeObject *object)
+{
+	g_mime_event_unblock (object->headers->changed, (GMimeEventCallback) header_list_changed, object);
 }
 
 static void
 content_type_changed (GMimeContentType *content_type, gpointer args, GMimeObject *object)
 {
-	GMimeParam *params;
-	GString *string;
-	char *type, *p;
+	char *raw_value;
 	
-	string = g_string_new ("Content-Type: ");
+	raw_value = g_mime_content_type_encode (content_type, NULL);
 	
-	type = g_mime_content_type_to_string (content_type);
-	g_string_append (string, type);
-	g_free (type);
-	
-	if ((params = content_type->params))
-		g_mime_param_write_to_string (params, FALSE, string);
-	
-	p = string->str;
-	g_string_free (string, FALSE);
-	
-	type = p + strlen ("Content-Type: ");
-	g_mime_header_list_set (object->headers, "Content-Type", type);
-	g_free (p);
-}
-
-static ssize_t
-write_disposition (GMimeStream *stream, const char *name, const char *value)
-{
-	GMimeContentDisposition *disposition;
-	ssize_t nwritten;
-	GString *out;
-	
-	out = g_string_new ("");
-	g_string_printf (out, "%s: ", name);
-	
-	disposition = g_mime_content_disposition_new_from_string (value);
-	g_string_append (out, disposition->disposition);
-	
-	g_mime_param_write_to_string (disposition->params, TRUE, out);
-	g_object_unref (disposition);
-	
-	nwritten = g_mime_stream_write (stream, out->str, out->len);
-	g_string_free (out, TRUE);
-	
-	return nwritten;
+	_g_mime_object_block_header_list_changed (object);
+	_g_mime_header_list_set (object->headers, "Content-Type", raw_value);
+	_g_mime_object_unblock_header_list_changed (object);
+	g_free (raw_value);
 }
 
 static void
 content_disposition_changed (GMimeContentDisposition *disposition, gpointer args, GMimeObject *object)
 {
-	char *str;
+	char *raw_value;
 	
-	if (object->disposition) {
-		str = g_mime_content_disposition_to_string (object->disposition, FALSE);
-		g_mime_header_list_set (object->headers, "Content-Disposition", str);
-		g_free (str);
+	_g_mime_object_block_header_list_changed (object);
+	
+	if (disposition) {
+		raw_value = g_mime_content_disposition_encode (disposition, NULL);
+		_g_mime_header_list_set (object->headers, "Content-Disposition", raw_value);
+		g_free (raw_value);
 	} else {
 		g_mime_header_list_remove (object->headers, "Content-Disposition");
 	}
+	
+	_g_mime_object_unblock_header_list_changed (object);
 }
 
 
@@ -292,6 +386,7 @@ g_mime_object_register_type (const char *type, const char *subtype, GType object
 
 /**
  * g_mime_object_new:
+ * @options: (nullable): a #GMimeParserOptions or %NULL
  * @content_type: a #GMimeContentType object
  *
  * Performs a lookup of registered #GMimeObject subclasses, registered
@@ -306,7 +401,7 @@ g_mime_object_register_type (const char *type, const char *subtype, GType object
  * parts appropriate for @content_type.
  **/
 GMimeObject *
-g_mime_object_new (GMimeContentType *content_type)
+g_mime_object_new (GMimeParserOptions *options, GMimeContentType *content_type)
 {
 	struct _type_bucket *bucket;
 	struct _subtype_bucket *sub;
@@ -336,7 +431,8 @@ g_mime_object_new (GMimeContentType *content_type)
 			return NULL;
 	}
 	
-	object = g_object_newv (obj_type, 0, NULL);
+	object = g_object_new (obj_type, NULL);
+	_g_mime_header_list_set_options (object->headers, options);
 	
 	g_mime_object_set_content_type (object, content_type);
 	
@@ -346,6 +442,7 @@ g_mime_object_new (GMimeContentType *content_type)
 
 /**
  * g_mime_object_new_type:
+ * @options: (nullable): a #GMimeParserOptions or %NULL
  * @type: mime type
  * @subtype: mime subtype
  *
@@ -360,10 +457,11 @@ g_mime_object_new (GMimeContentType *content_type)
  * of @type/@subtype.
  **/
 GMimeObject *
-g_mime_object_new_type (const char *type, const char *subtype)
+g_mime_object_new_type (GMimeParserOptions *options, const char *type, const char *subtype)
 {
 	struct _type_bucket *bucket;
 	struct _subtype_bucket *sub;
+	GMimeObject *object;
 	GType obj_type;
 	
 	g_return_val_if_fail (type != NULL, NULL);
@@ -389,7 +487,10 @@ g_mime_object_new_type (const char *type, const char *subtype)
 			return NULL;
 	}
 	
-	return g_object_newv (obj_type, 0, NULL);
+	object = g_object_new (obj_type, NULL);
+	_g_mime_header_list_set_options (object->headers, options);
+	
+	return object;
 }
 
 
@@ -397,11 +498,11 @@ static void
 object_set_content_type (GMimeObject *object, GMimeContentType *content_type)
 {
 	if (object->content_type) {
-		g_mime_event_remove (object->content_type->priv, (GMimeEventCallback) content_type_changed, object);
+		g_mime_event_remove (object->content_type->changed, (GMimeEventCallback) content_type_changed, object);
 		g_object_unref (object->content_type);
 	}
 	
-	g_mime_event_add (content_type->priv, (GMimeEventCallback) content_type_changed, object);
+	g_mime_event_add (content_type->changed, (GMimeEventCallback) content_type_changed, object);
 	object->content_type = content_type;
 	g_object_ref (content_type);
 }
@@ -527,7 +628,7 @@ g_mime_object_get_content_disposition (GMimeObject *object)
 }
 
 /**
- * g_mime_object_set_content_disposition:
+ * _g_mime_object_set_content_disposition:
  * @object: a #GMimeObject
  * @disposition: a #GMimeContentDisposition object
  *
@@ -541,11 +642,11 @@ static void
 _g_mime_object_set_content_disposition (GMimeObject *object, GMimeContentDisposition *disposition)
 {
 	if (object->disposition) {
-		g_mime_event_remove (object->disposition->priv, (GMimeEventCallback) content_disposition_changed, object);
+		g_mime_event_remove (object->disposition->changed, (GMimeEventCallback) content_disposition_changed, object);
 		g_object_unref (object->disposition);
 	}
 	
-	g_mime_event_add (disposition->priv, (GMimeEventCallback) content_disposition_changed, object);
+	g_mime_event_add (disposition->changed, (GMimeEventCallback) content_disposition_changed, object);
 	object->disposition = disposition;
 	g_object_ref (disposition);
 }
@@ -697,7 +798,9 @@ g_mime_object_set_content_id (GMimeObject *object, const char *content_id)
 	object->content_id = g_strdup (content_id);
 	
 	msgid = g_strdup_printf ("<%s>", content_id);
-	g_mime_object_set_header (object, "Content-Id", msgid);
+	_g_mime_object_block_header_list_changed (object);
+	g_mime_header_list_set (object->headers, "Content-Id", msgid, NULL);
+	_g_mime_object_unblock_header_list_changed (object);
 	g_free (msgid);
 }
 
@@ -719,92 +822,30 @@ g_mime_object_get_content_id (GMimeObject *object)
 }
 
 
-enum {
-	HEADER_CONTENT_DISPOSITION,
-	HEADER_CONTENT_TYPE,
-	HEADER_CONTENT_ID,
-	HEADER_UNKNOWN,
-};
-
-static char *content_headers[] = {
-	"Content-Disposition",
-	"Content-Type",
-	"Content-Id",
-};
-
-static gboolean
-process_header (GMimeObject *object, const char *header, const char *value)
-{
-	GMimeContentDisposition *disposition;
-	GMimeContentType *content_type;
-	guint i;
-	
-	if (g_ascii_strncasecmp (header, "Content-", 8) != 0)
-		return FALSE;
-	
-	for (i = 0; i < G_N_ELEMENTS (content_headers); i++) {
-		if (!g_ascii_strcasecmp (content_headers[i] + 8, header + 8))
-			break;
-	}
-	
-	switch (i) {
-	case HEADER_CONTENT_DISPOSITION:
-		disposition = g_mime_content_disposition_new_from_string (value);
-		_g_mime_object_set_content_disposition (object, disposition);
-		g_object_unref (disposition);
-		break;
-	case HEADER_CONTENT_TYPE:
-		content_type = g_mime_content_type_new_from_string (value);
-		_g_mime_object_set_content_type (object, content_type);
-		g_object_unref (content_type);
-		break;
-	case HEADER_CONTENT_ID:
-		g_free (object->content_id);
-		object->content_id = g_mime_utils_decode_message_id (value);
-		break;
-	default:
-		return FALSE;
-	}
-	
-	g_mime_header_list_set (object->headers, header, value);
-	
-	return TRUE;
-}
-
-static void
-object_prepend_header (GMimeObject *object, const char *header, const char *value)
-{
-	if (!process_header (object, header, value))
-		g_mime_header_list_prepend (object->headers, header, value);
-}
-
-
 /**
  * g_mime_object_prepend_header:
  * @object: a #GMimeObject
  * @header: header name
  * @value: header value
+ * @charset: a charset
  *
- * Prepends a raw, unprocessed header to the MIME object.
- *
- * Note: @value should be encoded with a function such as
- * g_mime_utils_header_encode_text().
+ * Prepends a new header to the header list.
  **/
 void
-g_mime_object_prepend_header (GMimeObject *object, const char *header, const char *value)
+g_mime_object_prepend_header (GMimeObject *object, const char *header, const char *value, const char *charset)
 {
 	g_return_if_fail (GMIME_IS_OBJECT (object));
 	g_return_if_fail (header != NULL);
-	g_return_if_fail (value != NULL);
 	
-	GMIME_OBJECT_GET_CLASS (object)->prepend_header (object, header, value);
+	g_mime_header_list_prepend (object->headers, header, value, charset);
 }
 
-static void
-object_append_header (GMimeObject *object, const char *header, const char *value)
+
+void
+_g_mime_object_append_header (GMimeObject *object, const char *header, const char *raw_name,
+			      const char *raw_value, gint64 offset)
 {
-	if (!process_header (object, header, value))
-		g_mime_header_list_append (object->headers, header, value);
+	_g_mime_header_list_append (object->headers, header, raw_name, raw_value, offset);
 }
 
 
@@ -813,28 +854,17 @@ object_append_header (GMimeObject *object, const char *header, const char *value
  * @object: a #GMimeObject
  * @header: header name
  * @value: header value
+ * @charset: a charset
  *
- * Appends a raw, unprocessed header to the MIME object.
- *
- * Note: @value should be encoded with a function such as
- * g_mime_utils_header_encode_text().
+ * Appends a new header to the header list.
  **/
 void
-g_mime_object_append_header (GMimeObject *object, const char *header, const char *value)
+g_mime_object_append_header (GMimeObject *object, const char *header, const char *value, const char *charset)
 {
 	g_return_if_fail (GMIME_IS_OBJECT (object));
 	g_return_if_fail (header != NULL);
-	g_return_if_fail (value != NULL);
 	
-	GMIME_OBJECT_GET_CLASS (object)->append_header (object, header, value);
-}
-
-
-static void
-object_set_header (GMimeObject *object, const char *header, const char *value)
-{
-	if (!process_header (object, header, value))
-		g_mime_header_list_set (object->headers, header, value);
+	g_mime_header_list_append (object->headers, header, value, charset);
 }
 
 
@@ -843,27 +873,17 @@ object_set_header (GMimeObject *object, const char *header, const char *value)
  * @object: a #GMimeObject
  * @header: header name
  * @value: header value
+ * @charset: a charset
  *
- * Sets an arbitrary raw, unprocessed header on the MIME object.
- *
- * Note: @value should be encoded with a function such as
- * g_mime_utils_header_encode_text().
+ * Sets a header to the specified value.
  **/
 void
-g_mime_object_set_header (GMimeObject *object, const char *header, const char *value)
+g_mime_object_set_header (GMimeObject *object, const char *header, const char *value, const char *charset)
 {
 	g_return_if_fail (GMIME_IS_OBJECT (object));
 	g_return_if_fail (header != NULL);
-	g_return_if_fail (value != NULL);
 	
-	GMIME_OBJECT_GET_CLASS (object)->set_header (object, header, value);
-}
-
-
-static const char *
-object_get_header (GMimeObject *object, const char *header)
-{
-	return g_mime_header_list_get (object->headers, header);
+	g_mime_header_list_set (object->headers, header, value, charset);
 }
 
 
@@ -872,54 +892,22 @@ object_get_header (GMimeObject *object, const char *header)
  * @object: a #GMimeObject
  * @header: header name
  *
- * Gets the raw, unprocessed value of the first header with the specified name.
+ * Gets the value of the first header with the specified name.
  *
- * Returns: the raw, unprocessed value of the requested header if it
+ * Returns: the value of the requested header if it
  * exists or %NULL otherwise.
- *
- * Note: The returned value should be decoded with a function such as
- * g_mime_utils_header_decode_text() before displaying to the user.
  **/
 const char *
 g_mime_object_get_header (GMimeObject *object, const char *header)
 {
+	GMimeHeader *hdr;
+	
 	g_return_val_if_fail (GMIME_IS_OBJECT (object), NULL);
-	g_return_val_if_fail (header != NULL, NULL);
 	
-	return GMIME_OBJECT_GET_CLASS (object)->get_header (object, header);
-}
-
-
-static gboolean
-object_remove_header (GMimeObject *object, const char *header)
-{
-	guint i;
+	if (!(hdr = g_mime_header_list_get_header (object->headers, header)))
+		return NULL;
 	
-	for (i = 0; i < G_N_ELEMENTS (content_headers); i++) {
-		if (!g_ascii_strcasecmp (content_headers[i], header))
-			break;
-	}
-	
-	switch (i) {
-	case HEADER_CONTENT_DISPOSITION:
-		if (object->disposition) {
-			g_mime_event_remove (object->disposition->priv, (GMimeEventCallback) content_disposition_changed, object);
-			g_object_unref (object->disposition);
-			object->disposition = NULL;
-		}
-		break;
-	case HEADER_CONTENT_TYPE:
-		/* never allow the removal of the Content-Type header */
-		return FALSE;
-	case HEADER_CONTENT_ID:
-		g_free (object->content_id);
-		object->content_id = NULL;
-		break;
-	default:
-		break;
-	}
-	
-	return g_mime_header_list_remove (object->headers, header);
+	return g_mime_header_get_value (hdr);
 }
 
 
@@ -937,22 +925,22 @@ gboolean
 g_mime_object_remove_header (GMimeObject *object, const char *header)
 {
 	g_return_val_if_fail (GMIME_IS_OBJECT (object), FALSE);
-	g_return_val_if_fail (header != NULL, FALSE);
 	
-	return GMIME_OBJECT_GET_CLASS (object)->remove_header (object, header);
+	return g_mime_header_list_remove (object->headers, header);
 }
 
 
 static char *
-object_get_headers (GMimeObject *object)
+object_get_headers (GMimeObject *object, GMimeFormatOptions *options)
 {
-	return g_mime_header_list_to_string (object->headers);
+	return g_mime_header_list_to_string (object->headers, options);
 }
 
 
 /**
  * g_mime_object_get_headers:
  * @object: a #GMimeObject
+ * @options: (nullable): a #GMimeFormatOptions or %NULL
  *
  * Allocates a string buffer containing all of the MIME object's raw
  * headers.
@@ -962,16 +950,16 @@ object_get_headers (GMimeObject *object)
  * Note: The returned string will not be suitable for display.
  **/
 char *
-g_mime_object_get_headers (GMimeObject *object)
+g_mime_object_get_headers (GMimeObject *object, GMimeFormatOptions *options)
 {
 	g_return_val_if_fail (GMIME_IS_OBJECT (object), NULL);
 	
-	return GMIME_OBJECT_GET_CLASS (object)->get_headers (object);
+	return GMIME_OBJECT_GET_CLASS (object)->get_headers (object, options);
 }
 
 
 static ssize_t
-object_write_to_stream (GMimeObject *object, GMimeStream *stream)
+object_write_to_stream (GMimeObject *object, GMimeFormatOptions *options, gboolean content_only, GMimeStream *stream)
 {
 	return -1;
 }
@@ -980,6 +968,7 @@ object_write_to_stream (GMimeObject *object, GMimeStream *stream)
 /**
  * g_mime_object_write_to_stream:
  * @object: a #GMimeObject
+ * @options: (nullable): a #GMimeFormatOptions or %NULL
  * @stream: stream
  *
  * Write the contents of the MIME object to @stream.
@@ -987,12 +976,12 @@ object_write_to_stream (GMimeObject *object, GMimeStream *stream)
  * Returns: the number of bytes written or %-1 on fail.
  **/
 ssize_t
-g_mime_object_write_to_stream (GMimeObject *object, GMimeStream *stream)
+g_mime_object_write_to_stream (GMimeObject *object, GMimeFormatOptions *options, GMimeStream *stream)
 {
 	g_return_val_if_fail (GMIME_IS_OBJECT (object), -1);
 	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
 	
-	return GMIME_OBJECT_GET_CLASS (object)->write_to_stream (object, stream);
+	return GMIME_OBJECT_GET_CLASS (object)->write_to_stream (object, options, FALSE, stream);
 }
 
 
@@ -1024,6 +1013,7 @@ g_mime_object_encode (GMimeObject *object, GMimeEncodingConstraint constraint)
 /**
  * g_mime_object_to_string:
  * @object: a #GMimeObject
+ * @options: (nullable): a #GMimeFormatOptions or %NULL
  *
  * Allocates a string buffer containing the contents of @object.
  *
@@ -1031,7 +1021,7 @@ g_mime_object_encode (GMimeObject *object, GMimeEncodingConstraint constraint)
  * object.
  **/
 char *
-g_mime_object_to_string (GMimeObject *object)
+g_mime_object_to_string (GMimeObject *object, GMimeFormatOptions *options)
 {
 	GMimeStream *stream;
 	GByteArray *array;
@@ -1043,7 +1033,7 @@ g_mime_object_to_string (GMimeObject *object)
 	stream = g_mime_stream_mem_new ();
 	g_mime_stream_mem_set_byte_array (GMIME_STREAM_MEM (stream), array);
 	
-	g_mime_object_write_to_stream (object, stream);
+	g_mime_object_write_to_stream (object, options, stream);
 	
 	g_object_unref (stream);
 	g_byte_array_append (array, (unsigned char *) "", 1);
