@@ -1526,7 +1526,7 @@ tokenize_rfc2047_phrase (GMimeParserOptions *options, const char *in, size_t *le
 					}
 					
 					/* sanity check encoding type */
-					if (inptr[0] != '?' || !strchr ("BbQq", inptr[1]) || inptr[2] != '?')
+					if (inptr[0] != '?' || inptr[1] == '\0' || !strchr ("BbQq", inptr[1]) || inptr[2] != '?')
 						goto non_rfc2047;
 					
 					inptr += 3;
@@ -1657,7 +1657,7 @@ tokenize_rfc2047_text (GMimeParserOptions *options, const char *in, size_t *len)
 					}
 					
 					/* sanity check encoding type */
-					if (inptr[0] != '?' || !strchr ("BbQq", inptr[1]) || inptr[2] != '?')
+					if (inptr[0] != '?' || inptr[1] == '\0' || !strchr ("BbQq", inptr[1]) || inptr[2] != '?')
 						goto non_rfc2047;
 					
 					inptr += 3;
@@ -1819,7 +1819,7 @@ rfc2047_decode_tokens (GMimeParserOptions *options, rfc2047_token *tokens, size_
 			
 			/* convert the raw decoded text into UTF-8 */
 			if (!g_ascii_strcasecmp (charset, "UTF-8")) {
-				/* slight optimization over going thru iconv */
+				/* slight optimization over going through iconv */
 				str = (char *) outptr;
 				len = outlen;
 				
@@ -2466,11 +2466,13 @@ header_fold_tokens (GMimeFormatOptions *options, const char *field, const char *
 	GString *output;
 	
 	len = strlen (field) + 2;
-	output = g_string_sized_new (len + vlen + 1);
+	
 	if (include_field) {
+		output = g_string_sized_new (len + vlen + 1);
 		g_string_append (output, field);
 		g_string_append (output, ": ");
 	} else {
+		output = g_string_sized_new (vlen + 2);
 		g_string_append_c (output, ' ');
 	}
 	lwsp = 0;
