@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -58,17 +58,21 @@ print_depth (GMimeStream *stream, int depth)
 static void
 print_mime_struct (GMimeStream *stream, GMimeObject *part, int depth)
 {
-	const GMimeContentType *type;
 	GMimeMultipart *multipart;
 	GMimeMessagePart *mpart;
+	GMimeContentType *type;
 	GMimeObject *subpart;
+	GMimeObject *body;
+	GMimeMessage *msg;
 	int i, n;
 	
 	print_depth (stream, depth);
 	
 	type = g_mime_object_get_content_type (part);
 	
-	g_mime_stream_printf (stream, "Content-Type: %s/%s\n", type->type, type->subtype);
+	g_mime_stream_printf (stream, "Content-Type: %s/%s\n",
+			      g_mime_content_type_get_media_type (type),
+			      g_mime_content_type_get_media_subtype (type));
 	
 	if (GMIME_IS_MULTIPART (part)) {
 		multipart = (GMimeMultipart *) part;
@@ -80,33 +84,37 @@ print_mime_struct (GMimeStream *stream, GMimeObject *part, int depth)
 		}
 	} else if (GMIME_IS_MESSAGE_PART (part)) {
 		mpart = (GMimeMessagePart *) part;
+		msg = g_mime_message_part_get_message (mpart);
 		
-		if (mpart->message)
-			print_mime_struct (stream, mpart->message->mime_part, depth + 1);
+		if (msg != NULL) {
+			body = g_mime_message_get_mime_part (msg);
+			
+			print_mime_struct (stream, body, depth + 1);
+		}
 	}
 }
 
-
 static void
-header_cb (GMimeParser *parser, const char *header, const char *value, gint64 offset, gpointer user_data)
+xevcb (GMimeParser *parser, const char *header, const char *value, gint64 offset, gpointer user_data)
 {
-	GMimeStream *stream = user_data;
-	
-	g_mime_stream_printf (stream, "%" G_GINT64_FORMAT ": %s: %s\n", offset, header, value);
 }
 
 static void
 test_parser (GMimeParser *parser, GMimeStream *mbox, GMimeStream *summary)
 {
-	gint64 message_begin, message_end, headers_begin, headers_end;
+	gint64 message_begin, message_end, headers_begin, headers_end, marker_offset;
+	GMimeFormatOptions *format = g_mime_format_options_get_default ();
+	InternetAddressList *list;
 	GMimeMessage *message;
-	const char *exev;
+	char *marker, *buf;
+	const char *subject;
+	GMimeObject *body;
+	GDateTime *date;
 	int nmsg = 0;
-	char *from;
 	
 	while (!g_mime_parser_eos (parser)) {
 		message_begin = g_mime_parser_tell (parser);
-		if (!(message = g_mime_parser_construct_message (parser)))
+		if (!(message = g_mime_parser_construct_message (parser, NULL)))
 			throw (exception_new ("failed to parse message #%d", nmsg));
 		
 		message_end = g_mime_parser_tell (parser);
@@ -119,20 +127,52 @@ test_parser (GMimeParser *parser, GMimeStream *mbox, GMimeStream *summary)
 		g_mime_stream_printf (summary, "header offsets: %" G_GINT64_FORMAT ", %" G_GINT64_FORMAT "\n",
 				      headers_begin, headers_end);
 		
-		from = g_mime_parser_get_from (parser);
-		g_mime_stream_printf (summary, "%s\n", from);
-		exev = g_mime_object_get_header ((GMimeObject *) message, "X-Evolution");
-		g_mime_stream_printf (summary, "X-Evolution: %s\n", exev ? exev : "None");
-		print_mime_struct (summary, message->mime_part, 0);
+		marker_offset = g_mime_parser_get_mbox_marker_offset (parser);
+		marker = g_mime_parser_get_mbox_marker (parser);
+		g_mime_stream_printf (summary, "%s\n", marker);
+		
+		if ((list = g_mime_message_get_from (message)) != NULL &&
+		    internet_address_list_length (list) > 0) {
+			buf = internet_address_list_to_string (list, format, FALSE);
+			g_mime_stream_printf (summary, "From: %s\n", buf);
+			g_free (buf);
+		}
+		
+		if ((list = g_mime_message_get_addresses (message, GMIME_ADDRESS_TYPE_TO)) != NULL &&
+		    internet_address_list_length (list) > 0) {
+			buf = internet_address_list_to_string (list, format, FALSE);
+			g_mime_stream_printf (summary, "To: %s\n", buf);
+			g_free (buf);
+		}
+		
+		if (!(subject = g_mime_message_get_subject (message)))
+			subject = "";
+		g_mime_stream_printf (summary, "Subject: %s\n", subject);
+		
+		if (!(date = g_mime_message_get_date (message))) {
+			date = g_date_time_new_from_unix_utc (0);
+		} else {
+			g_date_time_ref (date);
+		}
+		buf = g_mime_utils_header_format_date (date);
+		g_mime_stream_printf (summary, "Date: %s\n", buf);
+		g_date_time_unref (date);
+		g_free (buf);
+		
+		body = g_mime_message_get_mime_part (message);
+		print_mime_struct (summary, body, 0);
 		g_mime_stream_write (summary, "\n", 1);
 		
 		if (mbox) {
-			g_mime_stream_printf (mbox, "%s%s\n", nmsg > 0 ? "\n" : "", from);
-			g_mime_object_write_to_stream ((GMimeObject *) message, mbox);
+			if (nmsg > 0)
+				g_mime_stream_write (mbox, "\n", 1);
+			
+			g_mime_stream_printf (mbox, "%s\n", marker);
+			g_mime_object_write_to_stream ((GMimeObject *) message, format, mbox);
 		}
 		
 		g_object_unref (message);
-		g_free (from);
+		g_free (marker);
 		nmsg++;
 	}
 }
@@ -249,7 +289,7 @@ int main (int argc, char **argv)
 		return 0;
 #endif
 	
-	g_mime_init (0);
+	g_mime_init ();
 	
 	testsuite_init (argc, argv);
 	
@@ -300,23 +340,19 @@ int main (int argc, char **argv)
 			
 			testsuite_check ("%s", dent);
 			try {
-				if ((fd = open (input, O_RDONLY, 0)) == -1) {
+				if (!(istream = g_mime_stream_fs_open (input, O_RDONLY, 0, NULL))) {
 					throw (exception_new ("could not open `%s': %s",
 							      input, g_strerror (errno)));
 				}
 				
-				istream = g_mime_stream_fs_new (fd);
-				
-				if ((fd = open (output, O_RDONLY, 0)) == -1) {
+				if (!(ostream = g_mime_stream_fs_open (output, O_RDONLY, 0, NULL))) {
 					throw (exception_new ("could not open `%s': %s",
 							      output, g_strerror (errno)));
 				}
 				
-				ostream = g_mime_stream_fs_new (fd);
-				
 #ifdef ENABLE_MBOX_MATCH
-				tmp = g_strdup_printf ("./tmp/%s.XXXXXX", dent);
-				if ((fd = g_mkstemp (tmp)) == -1) {
+				tmp = g_strdup_printf ("./tmp/%s", dent);
+				if ((fd = open (tmp, O_CREAT | O_RDWR | O_TRUNC, 0644)) == -1) {
 					throw (exception_new ("could not open `%s': %s",
 							      tmp, g_strerror (errno)));
 				}
@@ -326,16 +362,33 @@ int main (int argc, char **argv)
 				
 				parser = g_mime_parser_new_with_stream (istream);
 				g_mime_parser_set_persist_stream (parser, TRUE);
-				g_mime_parser_set_scan_from (parser, TRUE);
+				g_mime_parser_set_format (parser, GMIME_FORMAT_MBOX);
 				
-				if (strstr (dent, "content-length") != NULL)
+				if (!g_mime_parser_get_persist_stream (parser))
+					throw (exception_new ("persist stream check failed"));
+				
+				if (g_mime_parser_get_format (parser) != GMIME_FORMAT_MBOX)
+					throw (exception_new ("format check failed"));
+				
+				if (strstr (dent, "content-length") != NULL) {
 					g_mime_parser_set_respect_content_length (parser, TRUE);
+					
+					if (!g_mime_parser_get_respect_content_length (parser))
+						throw (exception_new ("respect content-length check failed"));
+				} else {
+					g_mime_parser_set_respect_content_length (parser, FALSE);
+					
+					if (g_mime_parser_get_respect_content_length (parser))
+						throw (exception_new ("respect content-length check failed"));
+				}
+				
+				g_mime_parser_set_header_regex (parser, "^X-Evolution", xevcb, NULL);
 				
 				pstream = g_mime_stream_mem_new ();
-				g_mime_parser_set_header_regex (parser, "^Subject$", header_cb, pstream);
 				test_parser (parser, mstream, pstream);
 				
 #ifdef ENABLE_MBOX_MATCH
+				g_mime_stream_flush (mstream);
 				g_mime_stream_reset (istream);
 				g_mime_stream_reset (mstream);
 				if (!streams_match (istream, mstream))
@@ -380,12 +433,11 @@ int main (int argc, char **argv)
 		g_dir_close (dir);
 	} else if (S_ISREG (st.st_mode)) {
 		/* manually run test on a single file */
-		if ((fd = open (path, O_RDONLY, 0)) == -1)
+		if (!(istream = g_mime_stream_fs_open (path, O_RDONLY, 0, NULL)))
 			goto exit;
 		
-		istream = g_mime_stream_fs_new (fd);
 		parser = g_mime_parser_new_with_stream (istream);
-		g_mime_parser_set_scan_from (parser, TRUE);
+		g_mime_parser_set_format (parser, GMIME_FORMAT_MBOX);
 		
 #ifdef ENABLE_MBOX_MATCH
 		tmp = g_strdup ("./tmp/mbox-test.XXXXXX");
@@ -401,8 +453,8 @@ int main (int argc, char **argv)
 		mstream = NULL;
 #endif
 		
-		ostream = g_mime_stream_fs_new (dup (1));
-		g_mime_parser_set_header_regex (parser, "^Subject$", header_cb, ostream);
+		ostream = g_mime_stream_file_new (stdout);
+		g_mime_stream_file_set_owner ((GMimeStreamFile *) ostream, FALSE);
 		
 		testsuite_check ("user-input mbox: `%s'", path);
 		try {
@@ -437,19 +489,19 @@ int main (int argc, char **argv)
  exit:
 	
 #ifdef ENABLE_MBOX_MATCH
-	if ((dir = g_dir_open ("./tmp", 0, NULL))) {
-		p = g_stpcpy (input, "./tmp");
-		*p++ = G_DIR_SEPARATOR;
-		
-		while ((dent = g_dir_read_name (dir))) {
-			strcpy (p, dent);
-			unlink (input);
-		}
-		
-		g_dir_close (dir);
-	}
+	//if ((dir = g_dir_open ("./tmp", 0, NULL))) {
+	//	p = g_stpcpy (input, "./tmp");
+	//	*p++ = G_DIR_SEPARATOR;
+	//	
+	//	while ((dent = g_dir_read_name (dir))) {
+	//		strcpy (p, dent);
+	//		unlink (input);
+	//	}
+	//	
+	//	g_dir_close (dir);
+	//}
 	
-	rmdir ("./tmp");
+	//rmdir ("./tmp");
 #endif
 	
 	testsuite_end ();
