@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -63,173 +63,163 @@ uputs (const char *str, FILE *out)
 
 static struct {
 	const char *input;
+	const char *charset;
 	const char *display;
 	const char *encoded;
 } addrspec[] = {
-	{ "fejj@helixcode.com",
+	{ "fejj@helixcode.com", NULL,
 	  "fejj@helixcode.com",
 	  "fejj@helixcode.com" },
-	{ "this is\n\ta folded name <folded@name.com>",
-	  "this is a folded name <folded@name.com>",
-	  "this is a folded name <folded@name.com>" },
-	{ "Jeffrey Stedfast <fejj@helixcode.com>",
+	{ "Jeffrey Stedfast <fejj@helixcode.com>", NULL,
 	  "Jeffrey Stedfast <fejj@helixcode.com>",
 	  "Jeffrey Stedfast <fejj@helixcode.com>" },
-	{ "Jeffrey \"fejj\" Stedfast <fejj@helixcode.com>",
+	{ "Jeffrey \"fejj\" Stedfast <fejj@helixcode.com>", NULL,
 	  "Jeffrey fejj Stedfast <fejj@helixcode.com>",
 	  "Jeffrey fejj Stedfast <fejj@helixcode.com>" },
-	{ "\"Jeffrey \\\"fejj\\\" Stedfast\" <fejj@helixcode.com>",
+	{ "\"Jeffrey \\\"fejj\\\" Stedfast\" <fejj@helixcode.com>", NULL,
 	  "Jeffrey \"fejj\" Stedfast <fejj@helixcode.com>",
 	  "\"Jeffrey \\\"fejj\\\" Stedfast\" <fejj@helixcode.com>" },
-	{ "\"Stedfast, Jeffrey\" <fejj@helixcode.com>",
+	{ "\"Stedfast, Jeffrey\" <fejj@helixcode.com>", NULL,
 	  "\"Stedfast, Jeffrey\" <fejj@helixcode.com>",
 	  "\"Stedfast, Jeffrey\" <fejj@helixcode.com>" },
-	{ "fejj@helixcode.com (Jeffrey Stedfast)",
+	{ "fejj@helixcode.com (Jeffrey Stedfast)", NULL,
 	  "Jeffrey Stedfast <fejj@helixcode.com>",
 	  "Jeffrey Stedfast <fejj@helixcode.com>" },
-	{ "Jeff <fejj(recursive (comment) block)@helixcode.(and a comment here)com>",
+	{ "Jeff <fejj(recursive (comment) block)@helixcode.(and a comment here)com>", NULL,
 	  "Jeff <fejj@helixcode.com>",
 	  "Jeff <fejj@helixcode.com>" },
-	{ "=?iso-8859-1?q?Kristoffer_Br=E5nemyr?= <ztion@swipenet.se>",
+	{ "=?iso-8859-1?q?Kristoffer_Br=E5nemyr?= <ztion@swipenet.se>", "iso-8859-1",
 	  "Kristoffer Br\xc3\xa5nemyr <ztion@swipenet.se>",
 	  "Kristoffer =?iso-8859-1?q?Br=E5nemyr?= <ztion@swipenet.se>" },
-	{ "fpons@mandrakesoft.com (=?iso-8859-1?q?Fran=E7ois?= Pons)",
+	{ "fpons@mandrakesoft.com (=?iso-8859-1?q?Fran=E7ois?= Pons)", "iso-8859-1",
 	  "Fran\xc3\xa7ois Pons <fpons@mandrakesoft.com>",
 	  "=?iso-8859-1?q?Fran=E7ois?= Pons <fpons@mandrakesoft.com>" },
-	{ "GNOME Hackers: miguel@gnome.org (Miguel de Icaza), Havoc Pennington <hp@redhat.com>;, fejj@helixcode.com",
+	{ "GNOME Hackers: miguel@gnome.org (Miguel de Icaza), Havoc Pennington <hp@redhat.com>;, fejj@helixcode.com", NULL,
 	  "GNOME Hackers: Miguel de Icaza <miguel@gnome.org>, Havoc Pennington <hp@redhat.com>;, fejj@helixcode.com",
 	  "GNOME Hackers: Miguel de Icaza <miguel@gnome.org>, Havoc Pennington <hp@redhat.com>;, fejj@helixcode.com" },
-	{ "Local recipients: phil, joe, alex, bob",
+	{ "Local recipients: phil, joe, alex, bob", NULL,
 	  "Local recipients: phil, joe, alex, bob;",
 	  "Local recipients: phil, joe, alex, bob;" },
-	{ "\":sysmail\"@  Some-Group. Some-Org,\n Muhammed.(I am  the greatest) Ali @(the)Vegas.WBA",
+	{ "\":sysmail\"@  Some-Group. Some-Org,\n Muhammed.(I am  the greatest) Ali @(the)Vegas.WBA", NULL,
 	  "\":sysmail\"@Some-Group.Some-Org, Muhammed.Ali@Vegas.WBA",
 	  "\":sysmail\"@Some-Group.Some-Org, Muhammed.Ali@Vegas.WBA" },
-	{ "Charles S. Kerr <charles@foo.com>",
+	{ "Charles S. Kerr <charles@foo.com>", NULL,
 	  "\"Charles S. Kerr\" <charles@foo.com>",
 	  "\"Charles S. Kerr\" <charles@foo.com>" },
-	{ "Charles \"Likes, to, put, commas, in, quoted, strings\" Kerr <charles@foo.com>",
+	{ "Charles \"Likes, to, put, commas, in, quoted, strings\" Kerr <charles@foo.com>", NULL,
 	  "\"Charles Likes, to, put, commas, in, quoted, strings Kerr\" <charles@foo.com>",
 	  "\"Charles Likes, to, put, commas, in, quoted, strings Kerr\" <charles@foo.com>" },
-	{ "Charles Kerr, Pan Programmer <charles@superpimp.org>",
+	{ "Charles Kerr, Pan Programmer <charles@superpimp.org>", NULL,
 	  "\"Charles Kerr, Pan Programmer\" <charles@superpimp.org>",
 	  "\"Charles Kerr, Pan Programmer\" <charles@superpimp.org>" },
-	{ "Charles Kerr <charles@[127.0.0.1]>",
+	{ "Charles Kerr <charles@[127.0.0.1]>", NULL,
 	  "Charles Kerr <charles@[127.0.0.1]>",
 	  "Charles Kerr <charles@[127.0.0.1]>" },
-	{ "Charles <charles@[127..0.1]>",
-	  "Charles <charles@[127.0.1]>",
-	  "Charles <charles@[127.0.1]>" },
-	{ "Charles,, likes illegal commas <charles@superpimp.org>",
+	{ "Charles <charles@[127..0.1]>", NULL,
+	  "Charles <charles@[127..0.1]>",
+	  "Charles <charles@[127..0.1]>" },
+	{ "Charles,, likes illegal commas <charles@superpimp.org>", NULL,
 	  "Charles, likes illegal commas <charles@superpimp.org>",
 	  "Charles, likes illegal commas <charles@superpimp.org>" },
-	{ "<charles@>",
-	  "charles",
-	  "charles" },
-	{ "<charles@broken.host.com.> (Charles Kerr)",
-	  "Charles Kerr <charles@broken.host.com>",
-	  "Charles Kerr <charles@broken.host.com>" },
-	{ "fpons@mandrakesoft.com (=?iso-8859-1?q?Fran=E7ois?= Pons likes _'s and 	's too)",
+	{ "<charles@broken.host.com.>", NULL,
+	  "charles@broken.host.com",
+	  "charles@broken.host.com" },
+	{ "fpons@mandrakesoft.com (=?iso-8859-1?q?Fran=E7ois?= Pons likes _'s and 	's too)", "iso-8859-1",
 	  "\"Fran\xc3\xa7ois Pons likes _'s and 	's too\" <fpons@mandrakesoft.com>",
 	  "=?iso-8859-1?q?Fran=E7ois?= Pons likes _'s and 	's too <fpons@mandrakesoft.com>" },
-	{ "T\x81\xf5ivo Leedj\x81\xe4rv <leedjarv@interest.ee>",
+	{ "T\x81\xf5ivo Leedj\x81\xe4rv <leedjarv@interest.ee>", NULL,
 	  "T\xc2\x81\xc3\xb5ivo Leedj\xc2\x81\xc3\xa4rv <leedjarv@interest.ee>",
 	  "=?iso-8859-1?b?VIH1aXZvIExlZWRqgeRydg==?= <leedjarv@interest.ee>" },
-	{ "fbosi@mokabyte.it;, rspazzoli@mokabyte.it",
+	{ "fbosi@mokabyte.it;, rspazzoli@mokabyte.it", NULL,
 	  "fbosi@mokabyte.it, rspazzoli@mokabyte.it",
 	  "fbosi@mokabyte.it, rspazzoli@mokabyte.it" },
-	{ "\"Miles (Star Trekkin) O'Brian\" <mobrian@starfleet.org>",
+	{ "\"Miles (Star Trekkin) O'Brian\" <mobrian@starfleet.org>", NULL,
 	  "\"Miles (Star Trekkin) O'Brian\" <mobrian@starfleet.org>",
 	  "\"Miles (Star Trekkin) O'Brian\" <mobrian@starfleet.org>" },
-	{ "undisclosed-recipients: ;",
+	{ "undisclosed-recipients: ;", NULL,
 	  "undisclosed-recipients: ;",
 	  "undisclosed-recipients: ;" },
-	{ "undisclosed-recipients:;",
+	{ "undisclosed-recipients:;", NULL,
 	  "undisclosed-recipients: ;",
 	  "undisclosed-recipients: ;" },
-	{ "undisclosed-recipients:",
+	{ "undisclosed-recipients:", NULL,
 	  "undisclosed-recipients: ;",
 	  "undisclosed-recipients: ;" },
-	{ "undisclosed-recipients",
+	{ "undisclosed-recipients", NULL,
 	  "undisclosed-recipients",
 	  "undisclosed-recipients" },
-	/* The following 2 addr-specs are invalid according to the
-	 * spec, but apparently some japanese cellphones use them?
-	 * See Evolution bug #547969 */
-	{ "some...dots@hocus.pocus.net",
-	  "some...dots@hocus.pocus.net",
-	  "some...dots@hocus.pocus.net" },
-	{ "some.dots..@hocus.pocus.net",
-	  "some.dots..@hocus.pocus.net",
-	  "some.dots..@hocus.pocus.net" },
 	/* The following test case is to check that we properly handle
 	 * mailbox addresses that do not have any lwsp between the
 	 * name component and the addr-spec. See Evolution bug
 	 * #347520 */
-	{ "Canonical Patch Queue Manager<pqm@pqm.ubuntu.com>",
+	{ "Canonical Patch Queue Manager<pqm@pqm.ubuntu.com>", NULL,
 	  "Canonical Patch Queue Manager <pqm@pqm.ubuntu.com>",
 	  "Canonical Patch Queue Manager <pqm@pqm.ubuntu.com>" },
 	/* Some examples pulled from rfc5322 */
-	{ "Pete(A nice \\) chap) <pete(his account)@silly.test(his host)>",
+	{ "Pete(A nice \\) chap) <pete(his account)@silly.test(his host)>", NULL,
 	  "Pete <pete@silly.test>",
 	  "Pete <pete@silly.test>" },
-	{ "A Group(Some people):Chris Jones <c@(Chris's host.)public.example>, joe@example.org, John <jdoe@one.test> (my dear friend); (the end of the group)",
+	{ "A Group(Some people):Chris Jones <c@(Chris's host.)public.example>, joe@example.org, John <jdoe@one.test> (my dear friend); (the end of the group)", NULL,
 	  "A Group: Chris Jones <c@public.example>, joe@example.org, John <jdoe@one.test>;",
 	  "A Group: Chris Jones <c@public.example>, joe@example.org, John <jdoe@one.test>;" },
 	/* The following tests cases are meant to test forgivingness
 	 * of the parser when it encounters unquoted specials in the
 	 * name component */
-	{ "Warren Worthington, Jr. <warren@worthington.com>",
+	{ "Warren Worthington, Jr. <warren@worthington.com>", NULL,
 	  "\"Warren Worthington, Jr.\" <warren@worthington.com>",
 	  "\"Warren Worthington, Jr.\" <warren@worthington.com>" },
-	{ "dot.com <dot.com>",
+	{ "dot.com <dot.com>", NULL,
 	  "\"dot.com\" <dot.com>",
 	  "\"dot.com\" <dot.com>" },
-	{ "=?UTF-8?Q?agatest123_\"test\"?= <agatest123@o2.pl>",
+	{ "=?UTF-8?Q?agatest123_\"test\"?= <agatest123@o2.pl>", "utf-8",
 	  "agatest123 test <agatest123@o2.pl>",
 	  "agatest123 test <agatest123@o2.pl>" },
-	{ "\"=?ISO-8859-2?Q?TEST?=\" <p@p.org>",
+	{ "\"=?ISO-8859-2?Q?TEST?=\" <p@p.org>", "iso-8859-2",
 	  "TEST <p@p.org>",
 	  "TEST <p@p.org>" },
-	{ "sdfasf@wp.pl,c tert@wp.pl,sffdg.rtre@op.pl",
-	  "sdfasf@wp.pl, c, sffdg.rtre@op.pl",
-	  "sdfasf@wp.pl, c, sffdg.rtre@op.pl" },
+	{ "sdfasf@wp.pl,c tert@wp.pl,sffdg.rtre@op.pl", NULL,
+	  "sdfasf@wp.pl, sffdg.rtre@op.pl",
+	  "sdfasf@wp.pl, sffdg.rtre@op.pl" },
 	
 	/* obsolete routing address syntax tests */
-	{ "<@route:user@domain.com>",
+	{ "<@route:user@domain.com>", NULL,
 	  "user@domain.com",
 	  "user@domain.com" },
-	{ "<@route1,,@route2,,,@route3:user@domain.com>",
+	{ "<@route1,,@route2,,,@route3:user@domain.com>", NULL,
 	  "user@domain.com",
 	  "user@domain.com" },
 };
 
 static struct {
 	const char *input;
+	const char *charset;
 	const char *display;
 	const char *encoded;
 } broken_addrspec[] = {
-	{ "\"Biznes=?ISO-8859-2?Q?_?=INTERIA.PL\"=?ISO-8859-2?Q?_?=<biuletyny@firma.interia.pl>",
-	  "\"Biznes INTERIA.PL \" <biuletyny@firma.interia.pl>",
+	{ "\"Biznes=?ISO-8859-2?Q?_?=INTERIA.PL\"=?ISO-8859-2?Q?_?=<biuletyny@firma.interia.pl>", "iso-8859-2",
+	  "\"Biznes INTERIA.PL\" <biuletyny@firma.interia.pl>",
 	  "\"Biznes INTERIA.PL\" <biuletyny@firma.interia.pl>", },
 	/* UTF-8 sequence split between multiple encoded-word tokens */
-	{ "=?utf-8?Q?{#D=C3=A8=C3=A9=C2=A3=C3=A5=C3=BD_M$=C3=A1=C3?= =?utf-8?Q?=AD.=C3=A7=C3=B8m@#}?= <user@domain.com>",
-	  "\"{#Dèé£åý M$áí.çøm@#}\" <user@domain.com>",
-	  "=?iso-8859-1?b?eyNE6Omj5f0gTSTh7S7n+G1AI30=?= <user@domain.com>" },
+	{ "=?utf-8?Q?{#D=C3=A8=C3=A9=C2=A3=C3=A5=C3=BD_M$=C3=A1=C3?= =?utf-8?Q?=AD.=C3=A7=C3=B8m}?= <user@domain.com>", "utf-8",
+	  "\"{#Dèé£åý M$áí.çøm}\" <user@domain.com>",
+	  "=?UTF-8?b?eyNEw6jDqcKjw6XDvSBNJMOhw60uw6fDuG19?= <user@domain.com>" },
 	/* quoted-printable payload split between multiple encoded-word tokens */
-	{ "=?utf-8?Q?{#D=C3=A8=C3=A9=C2=?= =?utf-8?Q?A3=C3=A5=C3=BD_M$=C3=A1=C?= =?utf-8?Q?3=AD.=C3=A7=C3=B8m@#}?= <user@domain.com>",
-	  "\"{#Dèé£åý M$áí.çøm@#}\" <user@domain.com>",
-	  "=?iso-8859-1?b?eyNE6Omj5f0gTSTh7S7n+G1AI30=?= <user@domain.com>" },
+	{ "=?utf-8?Q?{#D=C3=A8=C3=A9=C2=?= =?utf-8?Q?A3=C3=A5=C3=BD_M$=C3=A1=C?= =?utf-8?Q?3=AD.=C3=A7=C3=B8m}?= <user@domain.com>", "utf-8",
+	  "\"{#Dèé£åý M$áí.çøm}\" <user@domain.com>",
+	  "=?UTF-8?b?eyNEw6jDqcKjw6XDvSBNJMOhw60uw6fDuG19?= <user@domain.com>" },
 	/* base64 payload split between multiple encoded-word tokens */
-	{ "=?iso-8859-1?b?ey?= =?iso-8859-1?b?NE6Omj5f0gTSTh7S7n+G1AI30=?= <user@domain.com>",
+	{ "=?iso-8859-1?b?ey?= =?iso-8859-1?b?NE6Omj5f0gTSTh7S7n+G1AI30=?= <user@domain.com>", "iso-8859-1",
 	  "\"{#Dèé£åý M$áí.çøm@#}\" <user@domain.com>",
 	  "=?iso-8859-1?b?eyNE6Omj5f0gTSTh7S7n+G1AI30=?= <user@domain.com>" },
 };
 
 static void
-test_addrspec (gboolean test_broken)
+test_addrspec (GMimeParserOptions *options, gboolean test_broken)
 {
+	GMimeFormatOptions *format = g_mime_format_options_get_default ();
 	InternetAddressList *addrlist;
+	InternetAddress *address;
+	const char *charset;
 	char *str;
 	guint i;
 	
@@ -239,17 +229,30 @@ test_addrspec (gboolean test_broken)
 		
 		testsuite_check ("addrspec[%u]", i);
 		try {
-			if (!(addrlist = internet_address_list_parse_string (addrspec[i].input)))
-				throw (exception_new ("could not parse addr-spec"));
+			if (!(addrlist = internet_address_list_parse (options, addrspec[i].input)))
+				throw (exception_new ("could not parse: %s", addrspec[i].input));
 			
-			str = internet_address_list_to_string (addrlist, FALSE);
+			if (!(address = internet_address_list_get_address (addrlist, 0)))
+				throw (exception_new ("could not get first address: %s", addrspec[i].input));
+			
+			charset = internet_address_get_charset (address);
+			if (addrspec[i].charset != NULL) {
+				if (charset == NULL)
+					throw (exception_new ("expected '%s' but got NULL charset: %s", addrspec[i].charset, addrspec[i].input));
+				if (g_ascii_strcasecmp (addrspec[i].charset, charset) != 0)
+					throw (exception_new ("expected '%s' but got '%s' charset: %s", addrspec[i].charset, charset, addrspec[i].input));
+			} else if (charset != NULL) {
+				throw (exception_new ("expected NULL charset but address has a charset of '%s': %s", charset, addrspec[i].input));
+			}
+			
+			str = internet_address_list_to_string (addrlist, format, FALSE);
 			if (strcmp (addrspec[i].display, str) != 0)
-				throw (exception_new ("display addr-spec %s does not match: %s", addrspec[i].display, str));
+				throw (exception_new ("display strings do not match.\ninput: %s\nexpected: %s\nactual: %s", addrspec[i].input, addrspec[i].display, str));
 			g_free (str);
 			
-			str = internet_address_list_to_string (addrlist, TRUE);
+			str = internet_address_list_to_string (addrlist, format, TRUE);
 			if (strcmp (addrspec[i].encoded, str) != 0)
-				throw (exception_new ("encoded addr-spec %s does not match: %s", addrspec[i].encoded, str));
+				throw (exception_new ("encoded strings do not match.\nexpected: %s\nactual: %s", addrspec[i].encoded, str));
 			
 			testsuite_check_passed ();
 		} catch (ex) {
@@ -268,17 +271,30 @@ test_addrspec (gboolean test_broken)
 			
 			testsuite_check ("broken_addrspec[%u]", i);
 			try {
-				if (!(addrlist = internet_address_list_parse_string (broken_addrspec[i].input)))
-					throw (exception_new ("could not parse addr-spec"));
+				if (!(addrlist = internet_address_list_parse (options, broken_addrspec[i].input)))
+					throw (exception_new ("could not parse: %s", broken_addrspec[i].input));
+
+				if (!(address = internet_address_list_get_address (addrlist, 0)))
+					throw (exception_new ("could not get first address: %s", broken_addrspec[i].input));
 				
-				str = internet_address_list_to_string (addrlist, FALSE);
+				charset = internet_address_get_charset (address);
+				if (broken_addrspec[i].charset != NULL) {
+					if (charset == NULL)
+						throw (exception_new ("expected '%s' but got NULL charset: %s", broken_addrspec[i].charset, broken_addrspec[i].input));
+					if (g_ascii_strcasecmp (broken_addrspec[i].charset, charset) != 0)
+						throw (exception_new ("expected '%s' but got '%s' charset: %s", broken_addrspec[i].charset, charset, broken_addrspec[i].input));
+				} else if (charset != NULL) {
+					throw (exception_new ("expected NULL charset but address has a charset of '%s': %s", charset, broken_addrspec[i].input));
+				}
+				
+				str = internet_address_list_to_string (addrlist, format, FALSE);
 				if (strcmp (broken_addrspec[i].display, str) != 0)
-					throw (exception_new ("display addr-spec %s does not match: %s", broken_addrspec[i].display, str));
+					throw (exception_new ("display strings do not match.\ninput: %s\nexpected: %s\nactual: %s", broken_addrspec[i].input, broken_addrspec[i].display, str));
 				g_free (str);
 				
-				str = internet_address_list_to_string (addrlist, TRUE);
+				str = internet_address_list_to_string (addrlist, format, TRUE);
 				if (strcmp (broken_addrspec[i].encoded, str) != 0)
-					throw (exception_new ("encoded addr-spec %s does not match: %s", broken_addrspec[i].encoded, str));
+					throw (exception_new ("encoded strings do not match.\nexpected: %s\nactual: %s", broken_addrspec[i].encoded, str));
 				
 				testsuite_check_passed ();
 			} catch (ex) {
@@ -320,39 +336,62 @@ static struct {
 	{ "17-6-2008 17:10:08",
 	  "Tue, 17 Jun 2008 17:10:08 +0000",
 	  1213722608, 0 },
+	{ "nonsense",
+	  "Thu, 01 Jan 1970 00:00:00 +0000",
+	  0, 0 }
 };
 
 static void
 test_date_parser (void)
 {
-	time_t date;
-	int tzone;
+	GDateTime *date;
+	Exception *ex;
+	int tz_offset;
+	GTimeSpan tz;
+	time_t time;
 	char *buf;
+	int sign;
 	guint i;
 	
 	for (i = 0; i < G_N_ELEMENTS (dates); i++) {
-		buf = NULL;
 		testsuite_check ("Date: '%s'", dates[i].in);
 		try {
-			date = g_mime_utils_header_decode_date (dates[i].in, &tzone);
+			if (!(date = g_mime_utils_header_decode_date (dates[i].in))) {
+				if (dates[i].date != 0)
+					throw (exception_new ("failed to parse date: %s", dates[i].in));
+				testsuite_check_passed ();
+				continue;
+			}
 			
-			if (date != dates[i].date)
-				throw (exception_new ("time_t's do not match"));
+			time = (time_t) g_date_time_to_unix (date);
+			tz = g_date_time_get_utc_offset (date);
 			
-			if (tzone != dates[i].tzone)
+			sign = tz < 0 ? -1 : 1;
+			tz *= sign;
+			
+			tz_offset = 100 * (tz / G_TIME_SPAN_HOUR);
+			tz_offset += (tz % G_TIME_SPAN_HOUR) / G_TIME_SPAN_MINUTE;
+			tz_offset *= sign;
+			
+			if (time != dates[i].date)
+				throw (exception_new ("time_t's do not match: %ld vs %ld", time, dates[i].date));
+			
+			if (tz_offset != dates[i].tzone)
 				throw (exception_new ("timezones do not match"));
 			
-			buf = g_mime_utils_header_format_date (date, tzone);
-			if (strcmp (dates[i].out, buf) != 0)
-				throw (exception_new ("date strings do not match"));
+			buf = g_mime_utils_header_format_date (date);
+			if (strcmp (dates[i].out, buf) != 0) {
+				ex = exception_new ("date strings do not match: %s vs %s", buf, dates[i].out);
+				g_free (buf);
+				throw (ex);
+			}
+			
+			g_free (buf);
 			
 			testsuite_check_passed ();
 		} catch (ex) {
-			testsuite_check_failed ("Date: '%s': %s", dates[i].in,
-						ex->message);
+			testsuite_check_failed ("Date: '%s': %s", dates[i].in, ex->message);
 		} finally;
-		
-		g_free (buf);
 	}
 }
 
@@ -427,8 +466,9 @@ static struct {
 #endif
 
 static void
-test_rfc2047 (gboolean test_broken)
+test_rfc2047 (GMimeParserOptions *options, gboolean test_broken)
 {
+	GMimeFormatOptions *format = g_mime_format_options_get_default ();
 	char *enc, *dec;
 	guint i;
 	
@@ -436,15 +476,15 @@ test_rfc2047 (gboolean test_broken)
 		dec = enc = NULL;
 		testsuite_check ("rfc2047_text[%u]", i);
 		try {
-			dec = g_mime_utils_header_decode_text (rfc2047_text[i].input);
+			dec = g_mime_utils_header_decode_text (options, rfc2047_text[i].input);
 			if (strcmp (rfc2047_text[i].decoded, dec) != 0)
 				throw (exception_new ("decoded text does not match: %s", dec));
 			
-			enc = g_mime_utils_header_encode_text (dec);
+			enc = g_mime_utils_header_encode_text (format, dec, NULL);
 			if (strcmp (rfc2047_text[i].encoded, enc) != 0)
-				throw (exception_new ("encoded text does not match: %s", enc));
+				throw (exception_new ("encoded text does not match: actual=\"%s\", expected=\"%s\"", enc, rfc2047_text[i].encoded));
 
-			//dec2 = g_mime_utils_header_decode_text (enc);
+			//dec2 = g_mime_utils_header_decode_text (options, enc);
 			//if (strcmp (rfc2047_text[i].decoded, dec2) != 0)
 			//	throw (exception_new ("decoded2 text does not match: %s", dec));
 			
@@ -461,11 +501,11 @@ test_rfc2047 (gboolean test_broken)
 		dec = enc = NULL;
 		testsuite_check ("broken_rfc2047_text[%u]", i);
 		try {
-			dec = g_mime_utils_header_decode_text (broken_rfc2047_text[i].input);
+			dec = g_mime_utils_header_decode_text (options, broken_rfc2047_text[i].input);
 			if (strcmp (broken_rfc2047_text[i].decoded, dec) != 0)
 				throw (exception_new ("decoded text does not match: %s", dec));
 			
-			enc = g_mime_utils_header_encode_text (dec);
+			enc = g_mime_utils_header_encode_text (format, dec, NULL);
 			if (strcmp (broken_rfc2047_text[i].encoded, enc) != 0)
 				throw (exception_new ("encoded text does not match: %s", enc));
 			
@@ -487,7 +527,7 @@ test_rfc2047 (gboolean test_broken)
 			if (strcmp (rfc2047_phrase[i].decoded, dec) != 0)
 				throw (exception_new ("decoded phrase does not match"));
 			
-			enc = g_mime_utils_header_encode_phrase (dec);
+			enc = g_mime_utils_header_encode_phrase (format, dec, NULL);
 			if (strcmp (rfc2047_phrase[i].encoded, enc) != 0)
 				throw (exception_new ("encoded phrase does not match"));
 			
@@ -511,8 +551,9 @@ static struct {
 };
 
 static void
-test_header_folding (void)
+test_header_folding (GMimeParserOptions *options)
 {
+	GMimeFormatOptions *format = g_mime_format_options_get_default ();
 	char *folded;
 	guint i;
 	
@@ -520,7 +561,7 @@ test_header_folding (void)
 	        folded = NULL;
 		testsuite_check ("header_folding[%u]", i);
 		try {
-			folded = g_mime_utils_unstructured_header_fold (header_folding[i].input);
+			folded = g_mime_utils_unstructured_header_fold (options, format, header_folding[i].input);
 			if (strcmp (header_folding[i].folded, folded) != 0)
 				throw (exception_new ("folded text does not match: -->%s<-- vs -->%s<--", header_folding[i].folded, folded));
 			
@@ -535,48 +576,94 @@ test_header_folding (void)
 
 static struct {
 	const char *input;
+	const char *charset;
 	const char *encoded;
+	GMimeParamEncodingMethod method;
 } rfc2184[] = {
 	{ "this is a really really long filename that should force gmime to rfc2184 encode it - yay!.html",
+	  "us-ascii",
 	  "Content-Disposition: attachment;\n\t"
-	  "filename*0*=iso-8859-1''this%20is%20a%20really%20really%20long%20filename%20;\n\t"
+	  "filename*0*=us-ascii'en'this%20is%20a%20really%20really%20long%20filename%20;\n\t"
 	  "filename*1*=that%20should%20force%20gmime%20to%20rfc2184%20encode%20it%20-;\n\t"
-	  "filename*2*=%20yay!.html\n" },
+	  "filename*2*=%20yay!.html\n",
+	  GMIME_PARAM_ENCODING_METHOD_RFC2231 },
+	{ "{#Dèé£åý M$áí.çøm}.doc",
+	  "iso-8859-1",
+	  "Content-Disposition: attachment;\n\t"
+	  "filename*=iso-8859-1'en'{#D%E8%E9%A3%E5%FD%20M$%E1%ED.%E7%F8m}.doc\n",
+	  GMIME_PARAM_ENCODING_METHOD_RFC2231 },
+	
+	/* Note: technically these aren't rfc2184-encoded... but they need to be parsed... */
+	{ "{#Dèé£åý M$áí.çøm}.doc",
+	  "iso-8859-1",
+	  "Content-Disposition: attachment;\n\t"
+	  "filename=\"=?iso-8859-1?b?eyNE6Omj5f0gTSTh7S7n+G19LmRvYw==?=\"\n",
+	  GMIME_PARAM_ENCODING_METHOD_RFC2047 },
 };
 
 static void
-test_rfc2184 (void)
+test_rfc2184 (GMimeParserOptions *options)
 {
-	GMimeParam param, *params;
+	GMimeFormatOptions *format = g_mime_format_options_get_default ();
+	GMimeParamEncodingMethod method;
+	GMimeParamList *params;
+	GMimeParam *param;
+	const char *value;
 	GString *str;
+	int count;
 	size_t n;
 	guint i;
 	
-	param.next = NULL;
-	param.name = "filename";
-	
-	str = g_string_new ("Content-Disposition: attachment");
-	n = str->len;
-	
 	for (i = 0; i < G_N_ELEMENTS (rfc2184); i++) {
-		params = NULL;
+		params = g_mime_param_list_new ();
+		g_mime_param_list_set_parameter (params, "filename", rfc2184[i].input);
+		param = g_mime_param_list_get_parameter (params, "filename");
+		g_mime_param_set_encoding_method (param, rfc2184[i].method);
+		g_mime_param_set_lang (param, "en");
 		
-		param.value = (char *) rfc2184[i].input;
+		str = g_string_new ("Content-Disposition: attachment");
+		n = str->len;
+		
+		g_mime_param_list_encode (params, format, TRUE, str);
+		g_object_unref (params);
+		params = NULL;
 		
 		testsuite_check ("rfc2184[%u]", i);
 		try {
-			g_mime_param_write_to_string (&param, TRUE, str);
 			if (strcmp (rfc2184[i].encoded, str->str) != 0)
-				throw (exception_new ("encoded param does not match: %s", str->str));
+				throw (exception_new ("encoded param list does not match: \n%s\nvs\n%s",
+						      rfc2184[i].encoded, str->str));
 			
-			if (!(params = g_mime_param_new_from_string (str->str + n + 2)))
+			if (!(params = g_mime_param_list_parse (options, str->str + n + 2)))
 				throw (exception_new ("could not parse encoded param list"));
 			
-			if (params->next != NULL)
-				throw (exception_new ("parsed more than a single param?"));
+			if ((count = g_mime_param_list_length (params)) != 1)
+				throw (exception_new ("expected only 1 param, but parsed %d", count));
 			
-			if (strcmp (rfc2184[i].input, params->value) != 0)
-				throw (exception_new ("parsed param value does not match"));
+			if (!(param = g_mime_param_list_get_parameter (params, "filename")))
+				throw (exception_new ("failed to get filename param"));
+			
+			value = g_mime_param_get_value (param);
+			if (strcmp (rfc2184[i].input, value) != 0)
+				throw (exception_new ("parsed param value does not match: %s", value));
+			
+			if (!(value = g_mime_param_get_charset (param)))
+				throw (exception_new ("parsed charset is NULL"));
+			
+			if (strcmp (rfc2184[i].charset, value) != 0)
+				throw (exception_new ("parsed charset does not match: %s", value));
+			
+			if (rfc2184[i].method != GMIME_PARAM_ENCODING_METHOD_RFC2047) {
+				if (!(value = g_mime_param_get_lang (param)))
+					throw (exception_new ("parsed lang is NULL"));
+				
+				if (strcmp (value, "en") != 0)
+					throw (exception_new ("parsed lang does not match: %s", value));
+			}
+			
+			method = g_mime_param_get_encoding_method (param);
+			if (method != rfc2184[i].method)
+				throw (exception_new ("parsed encoding method does not match: %d", method));
 			
 			testsuite_check_passed ();
 		} catch (ex) {
@@ -584,12 +671,10 @@ test_rfc2184 (void)
 		} finally;
 		
 		if (params != NULL)
-			g_mime_param_destroy (params);
+			g_object_unref (params);
 		
-		g_string_truncate (str, n);
+		g_string_free (str, TRUE);
 	}
-	
-	g_string_free (str, TRUE);
 }
 
 
@@ -640,40 +725,121 @@ test_qstring (void)
 }
 
 
+static struct {
+	const char *input;
+	int count;
+	const char *ids[5];
+} references[] = {
+	{ "<3ohapq$h3b@gandalf.rutgers.edu> <3notqh$b52@ns2.ny.ubs.com> <3npoh0$2oo@news.blkbox.com> <3nqp09$r7t@ns2.ny.ubs.com>",
+	  4,
+	  { "3ohapq$h3b@gandalf.rutgers.edu", "3notqh$b52@ns2.ny.ubs.com", "3npoh0$2oo@news.blkbox.com", "3nqp09$r7t@ns2.ny.ubs.com", NULL }
+	},
+	{ "<3lmtu0$dv1@secnews.netscape.com> <3lpjth$g97@secnews.netscape.com> <3lrbuf$gvp@secnews.netscape.com> <3lst13$iur@secnews.netscape.com>",
+	  4,
+	  { "3lmtu0$dv1@secnews.netscape.com", "3lpjth$g97@secnews.netscape.com", "3lrbuf$gvp@secnews.netscape.com", "3lst13$iur@secnews.netscape.com", NULL }
+	},
+};
+
+
+static void
+test_references (GMimeParserOptions *options)
+{
+	GMimeReferences *refs, *copy;
+	guint i;
+	int j;
+	
+	for (i = 0; i < G_N_ELEMENTS (references); i++) {
+		testsuite_check ("references[%u]", i);
+		try {
+			if (!(refs = g_mime_references_parse (options, references[i].input)))
+				throw (exception_new ("failed to parse references"));
+			
+			if (g_mime_references_length (refs) != references[i].count)
+				throw (exception_new ("number of references does not match"));
+			
+			for (j = 0; j < references[i].count; j++) {
+				const char *msgid = g_mime_references_get_message_id (refs, j);
+				
+				if (strcmp (references[i].ids[j], msgid) != 0)
+					throw (exception_new ("message ids do not match for ids[%d]", j));
+			}
+			
+			copy = g_mime_references_copy (refs);
+			if (g_mime_references_length (copy) != g_mime_references_length (refs))
+				throw (exception_new ("number of copied references does not match"));
+			
+			for (j = 0; j < references[i].count; j++) {
+				const char *msgid = g_mime_references_get_message_id (copy, j);
+				
+				if (strcmp (references[i].ids[j], msgid) != 0)
+					throw (exception_new ("copied message ids do not match for ids[%d]", j));
+			}
+			
+			g_mime_references_clear (copy);
+			
+			if (g_mime_references_length (copy) != 0)
+				throw (exception_new ("references were not cleared"));
+			
+			g_mime_references_free (copy);
+			g_mime_references_free (refs);
+			
+			testsuite_check_passed ();
+		} catch (ex) {
+			testsuite_check_failed ("references[%u]: %s", i, ex->message);
+		} finally;
+	}
+}
+
 int main (int argc, char **argv)
 {
-	g_mime_init (0);
+	GMimeParserOptions *options = g_mime_parser_options_new ();
+	
+	g_mime_init ();
 	
 	testsuite_init (argc, argv);
 	
-	testsuite_start ("addr-spec parser");
-	test_addrspec (FALSE);
+	testsuite_start ("addr-spec parser (strict)");
+	g_mime_parser_options_set_rfc2047_compliance_mode (options, GMIME_RFC_COMPLIANCE_STRICT);
+	test_addrspec (options, FALSE);
+	testsuite_end ();
+	
+	testsuite_start ("addr-spec parser (loose)");
+	g_mime_parser_options_set_rfc2047_compliance_mode (options, GMIME_RFC_COMPLIANCE_LOOSE);
+	test_addrspec (options, TRUE);
 	testsuite_end ();
 	
 	testsuite_start ("date parser");
 	test_date_parser ();
 	testsuite_end ();
 	
-	testsuite_start ("rfc2047 encoding/decoding");
-	test_rfc2047 (FALSE);
+	testsuite_start ("rfc2047 encoding/decoding (strict)");
+	g_mime_parser_options_set_rfc2047_compliance_mode (options, GMIME_RFC_COMPLIANCE_STRICT);
+	test_rfc2047 (options, FALSE);
+	testsuite_end ();
+	
+	testsuite_start ("rfc2047 encoding/decoding (loose)");
+	g_mime_parser_options_set_rfc2047_compliance_mode (options, GMIME_RFC_COMPLIANCE_LOOSE);
+	test_rfc2047 (options, TRUE);
 	testsuite_end ();
 	
 	testsuite_start ("rfc2184 encoding/decoding");
-	test_rfc2184 ();
+	test_rfc2184 (options);
 	testsuite_end ();
 	
 	testsuite_start ("quoted-strings");
 	test_qstring ();
 	testsuite_end ();
 	
-	g_mime_shutdown ();
-	
-	g_mime_init (GMIME_ENABLE_RFC2047_WORKAROUNDS);
-	testsuite_start ("broken rfc2047 encoding/decoding");
-	test_header_folding ();
-	test_addrspec (TRUE);
-	test_rfc2047 (TRUE);
+	testsuite_start ("header folding");
+	test_header_folding (options);
 	testsuite_end ();
+	
+	testsuite_start ("references");
+	test_references (options);
+	testsuite_end ();
+	
+	g_mime_parser_options_free (options);
+	
 	g_mime_shutdown ();
 	
 	return testsuite_exit ();
