@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2013 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
  *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -41,64 +42,59 @@ extern int verbose;
 
 #define v(x) if (verbose > 3) x
 
+#if 0
 static gboolean
-request_passwd (GMimeCryptoContext *ctx, const char *user_id, const char *prompt_ctx, gboolean reprompt, GMimeStream *response, GError **err)
+request_passwd (GMimeCryptoContext *ctx, const char *user_id, const char *prompt, gboolean reprompt, GMimeStream *response, GError **err)
 {
 	g_mime_stream_write_string (response, "no.secret\n");
 	
 	return TRUE;
 }
+#endif
 
 static GMimeSignatureStatus
 get_sig_status (GMimeSignatureList *signatures)
 {
-	GMimeSignatureStatus status = GMIME_SIGNATURE_STATUS_GOOD;
+	GMimeSignatureStatus status = 0;
 	GMimeSignature *sig;
 	int i;
 	
 	if (!signatures || signatures->array->len == 0)
-		return GMIME_SIGNATURE_STATUS_ERROR;
+		return GMIME_SIGNATURE_STATUS_RED;
 	
 	for (i = 0; i < g_mime_signature_list_length (signatures); i++) {
 		sig = g_mime_signature_list_get_signature (signatures, i);
-		status = MAX (status, sig->status);
+		status |= g_mime_signature_get_status (sig);
 	}
 	
 	return status;
 }
 
 static void
-test_sign (GMimeCryptoContext *ctx, GMimeStream *cleartext, GMimeStream *ciphertext)
+test_sign (GMimeCryptoContext *ctx, gboolean detached, GMimeStream *cleartext, GMimeStream *ciphertext)
 {
 	GError *err = NULL;
 	Exception *ex;
 	int rv;
 	
-	rv = g_mime_crypto_context_sign (ctx, "alice@example.net",
-					 GMIME_DIGEST_ALGO_SHA256,
-					 cleartext, ciphertext, &err);
+	rv = g_mime_crypto_context_sign (ctx, detached, "mimekit@example.com", cleartext, ciphertext, &err);
 	
 	if (rv == -1 || err != NULL) {
 		ex = exception_new ("%s", err->message);
 		g_error_free (err);
 		throw (ex);
 	}
-	
-	v(fprintf (stderr, "signature (%s):\n%.*s\n",
-		   g_mime_crypto_context_digest_name (ctx, rv),
-		   GMIME_STREAM_MEM (ciphertext)->buffer->len,
-		   GMIME_STREAM_MEM (ciphertext)->buffer->data));
 }
 
 static void
-test_verify (GMimeCryptoContext *ctx, GMimeStream *cleartext, GMimeStream *ciphertext)
+test_verify_detached (GMimeCryptoContext *ctx, GMimeStream *cleartext, GMimeStream *ciphertext)
 {
 	GMimeSignatureList *signatures;
+	GMimeSignatureStatus status;
 	GError *err = NULL;
 	Exception *ex;
 	
-	signatures = g_mime_crypto_context_verify (ctx, GMIME_DIGEST_ALGO_DEFAULT,
-						   cleartext, ciphertext, &err);
+	signatures = g_mime_crypto_context_verify (ctx, 0, cleartext, ciphertext, NULL, &err);
 	
 	if (signatures == NULL) {
 		ex = exception_new ("%s", err->message);
@@ -106,7 +102,9 @@ test_verify (GMimeCryptoContext *ctx, GMimeStream *cleartext, GMimeStream *ciphe
 		throw (ex);
 	}
 	
-	if (get_sig_status (signatures) != GMIME_SIGNATURE_STATUS_GOOD) {
+	status = get_sig_status (signatures);
+	
+	if ((status & GMIME_SIGNATURE_STATUS_RED) != 0) {
 		g_object_unref (signatures);
 		throw (exception_new ("signature BAD"));
 	}
@@ -115,18 +113,61 @@ test_verify (GMimeCryptoContext *ctx, GMimeStream *cleartext, GMimeStream *ciphe
 }
 
 static void
-test_encrypt (GMimeCryptoContext *ctx, gboolean sign, GMimeStream *cleartext, GMimeStream *ciphertext)
+test_verify (GMimeCryptoContext *ctx, GMimeStream *cleartext, GMimeStream *ciphertext)
+{
+	GMimeSignatureList *signatures;
+	GMimeSignatureStatus status;
+	Exception *ex = NULL;
+	GMimeStream *stream;
+	GError *err = NULL;
+	GByteArray *buf[2];
+	
+	stream = g_mime_stream_mem_new ();
+	
+	signatures = g_mime_crypto_context_verify (ctx, 0, ciphertext, NULL, stream, &err);
+	
+	if (signatures == NULL) {
+		ex = exception_new ("%s", err->message);
+		g_object_unref (stream);
+		g_error_free (err);
+		throw (ex);
+	}
+	
+	status = get_sig_status (signatures);
+	
+	if ((status & GMIME_SIGNATURE_STATUS_RED) != 0) {
+		g_object_unref (signatures);
+		g_object_unref (stream);
+		
+		throw (exception_new ("signature BAD"));
+	}
+	
+	g_object_unref (signatures);
+	
+	buf[0] = GMIME_STREAM_MEM (cleartext)->buffer;
+	buf[1] = GMIME_STREAM_MEM (stream)->buffer;
+	
+	if (buf[0]->len != buf[1]->len || memcmp (buf[0]->data, buf[1]->data, buf[0]->len) != 0)
+		ex = exception_new ("extracted data does not match original cleartext");
+	
+	g_object_unref (stream);
+	
+	if (ex != NULL)
+		throw (ex);
+}
+
+static void
+test_encrypt (GMimeCryptoContext *ctx, GMimeStream *cleartext, GMimeStream *ciphertext)
 {
 	GPtrArray *recipients;
 	GError *err = NULL;
 	Exception *ex;
 	
 	recipients = g_ptr_array_new ();
-	g_ptr_array_add (recipients, "alice@example.net");
+	g_ptr_array_add (recipients, "mimekit@example.com");
 	
-	g_mime_crypto_context_encrypt (ctx, sign, "alice@example.net",
-				       GMIME_DIGEST_ALGO_SHA256, recipients,
-				       cleartext, ciphertext, &err);
+	g_mime_crypto_context_encrypt (ctx, FALSE, NULL, GMIME_ENCRYPT_NONE,
+				       recipients, cleartext, ciphertext, &err);
 	
 	g_ptr_array_free (recipients, TRUE);
 	
@@ -135,15 +176,12 @@ test_encrypt (GMimeCryptoContext *ctx, gboolean sign, GMimeStream *cleartext, GM
 		g_error_free (err);
 		throw (ex);
 	}
-	
-	v(fprintf (stderr, "ciphertext:\n%.*s\n",
-		   GMIME_STREAM_MEM (ciphertext)->buffer->len,
-		   GMIME_STREAM_MEM (ciphertext)->buffer->data));
 }
 
 static void
-test_decrypt (GMimeCryptoContext *ctx, gboolean sign, GMimeStream *cleartext, GMimeStream *ciphertext)
+test_decrypt (GMimeCryptoContext *ctx, GMimeStream *cleartext, GMimeStream *ciphertext)
 {
+	GMimeCertificateList *recipients;
 	GMimeDecryptResult *result;
 	Exception *ex = NULL;
 	GMimeStream *stream;
@@ -152,19 +190,18 @@ test_decrypt (GMimeCryptoContext *ctx, gboolean sign, GMimeStream *cleartext, GM
 	
 	stream = g_mime_stream_mem_new ();
 	
-	if (!(result = g_mime_crypto_context_decrypt (ctx, ciphertext, stream, &err))) {
+	if (!(result = g_mime_crypto_context_decrypt (ctx, 0, NULL, ciphertext, stream, &err))) {
 		g_object_unref (stream);
 		ex = exception_new ("%s", err->message);
 		g_error_free (err);
 		throw (ex);
 	}
 	
-	if (sign) {
-		if (!result->signatures || get_sig_status (result->signatures) != GMIME_SIGNATURE_STATUS_GOOD)
-			ex = exception_new ("expected GOOD signature");
-	} else {
-		if (result->signatures)
-			ex = exception_new ("unexpected signature");
+	if (!(recipients = g_mime_decrypt_result_get_recipients (result))) {
+		g_object_unref (result);
+		g_object_unref (stream);
+		
+		throw (exception_new ("Failed to get recipients"));
 	}
 	
 	g_object_unref (result);
@@ -194,8 +231,8 @@ test_export (GMimeCryptoContext *ctx, const char *path)
 	const char *inbuf, *outbuf;
 	size_t inlen, outlen;
 	Exception *ex = NULL;
+	const char *keys[2];
 	GError *err = NULL;
-	GPtrArray *keys;
 	int fd;
 	
 	if ((fd = open (path, O_RDONLY, 0)) == -1)
@@ -207,13 +244,13 @@ test_export (GMimeCryptoContext *ctx, const char *path)
 	g_mime_stream_reset (istream);
 	g_object_unref (ostream);
 	
-	keys = g_ptr_array_new ();
-	g_ptr_array_add (keys, "alice@example.net");
+	keys[0] = "mimekit@example.com";
+	keys[1] = NULL;
 	
 	ostream = g_mime_stream_mem_new ();
 	
 	g_mime_crypto_context_export_keys (ctx, keys, ostream, &err);
-	g_ptr_array_free (keys, TRUE);
+	
 	if (err != NULL) {
 		ex = exception_new ("%s", err->message);
 		g_object_unref (istream);
@@ -224,24 +261,18 @@ test_export (GMimeCryptoContext *ctx, const char *path)
 	
 	inbuf = (const char *) GMIME_STREAM_MEM (istream)->buffer->data;
 	inlen = GMIME_STREAM_MEM (istream)->buffer->len;
-	if ((inptr = strstr (inbuf, "\n\n"))) {
-		/* skip past the headers which may have different version numbers */
-		inptr += 2;
-		inlen -= (inptr - inbuf);
-		inbuf = inptr;
-	}
 	
 	outbuf = (const char *) GMIME_STREAM_MEM (ostream)->buffer->data;
 	outlen = GMIME_STREAM_MEM (ostream)->buffer->len;
-	if (outbuf && (inptr = strstr (outbuf, "\n\n"))) {
-		/* skip past the headers which may have different version numbers */
-		inptr += 2;
-		outlen -= (inptr - outbuf);
-		outbuf = inptr;
-	}
 	
-	if (outlen != inlen || memcmp (outbuf, inbuf, inlen) != 0)
+	if (outlen != inlen || memcmp (outbuf, inbuf, inlen) != 0) {
+		FILE *fp = fopen ("exported.crt", "w");
+		fwrite (outbuf, 1, outlen, fp);
+		fflush (fp);
+		fclose (fp);
+		
 		ex = exception_new ("exported key does not match original key");
+	}
 	
 	g_object_unref (istream);
 	g_object_unref (ostream);
@@ -275,7 +306,7 @@ import_key (GMimeCryptoContext *ctx, const char *path)
 
 int main (int argc, char **argv)
 {
-	const char *datadir = "data/pkcs7";
+	const char *datadir = "data/smime";
 	GMimeStream *istream, *ostream;
 	GMimeCryptoContext *ctx;
 	const char *what;
@@ -283,16 +314,12 @@ int main (int argc, char **argv)
 	char *key;
 	int i;
 	
-	g_mime_init (0);
+	g_mime_init ();
 	
 	testsuite_init (argc, argv);
 	
-	/* reset .gnupg config directory */
-	if (system ("/bin/rm -rf ./tmp") != 0)
+	if (testsuite_setup_gpghome ("gpgsm") != 0)
 		return EXIT_FAILURE;
-	if (system ("/bin/mkdir ./tmp") != 0)
-		return EXIT_FAILURE;
-	g_setenv ("GNUPGHOME", "./tmp/.gnupg", 1);
 	
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] != '-') {
@@ -306,17 +333,17 @@ int main (int argc, char **argv)
 	
 	testsuite_start ("Pkcs7 crypto context");
 	
-	ctx = g_mime_pkcs7_context_new (request_passwd);
-	g_mime_pkcs7_context_set_always_trust ((GMimePkcs7Context *) ctx, TRUE);
+	ctx = g_mime_pkcs7_context_new ();
+	//g_mime_crypto_context_set_request_password (ctx, request_passwd);
 	
 	testsuite_check ("GMimePkcs7Context::import");
 	try {
-		key = g_build_filename (datadir, "alice.pem", NULL);
+		key = g_build_filename (datadir, "certificate-authority.crt", NULL);
 		//printf ("importing key: %s\n", key);
 		import_key (ctx, key);
 		g_free (key);
 		
-		key = g_build_filename (datadir, "alice.p12", NULL);
+		key = g_build_filename (datadir, "smime.p12", NULL);
 		//printf ("importing key: %s\n", key);
 		import_key (ctx, key);
 		g_free (key);
@@ -327,7 +354,7 @@ int main (int argc, char **argv)
 		return EXIT_FAILURE;
 	} finally;
 	
-	key = g_build_filename (datadir, "alice.pem", NULL);
+	key = g_build_filename (datadir, "smime.crt", NULL);
 	testsuite_check ("GMimePkcs7Context::export");
 	try {
 		test_export (ctx, key);
@@ -347,7 +374,7 @@ int main (int argc, char **argv)
 	what = "GMimePkcs7Context::sign";
 	testsuite_check (what);
 	try {
-		test_sign (ctx, istream, ostream);
+		test_sign (ctx, FALSE, istream, ostream);
 		testsuite_check_passed ();
 		
 		what = "GMimePkcs7Context::verify";
@@ -364,17 +391,17 @@ int main (int argc, char **argv)
 	g_mime_stream_reset (istream);
 	ostream = g_mime_stream_mem_new ();
 	
-	what = "GMimePkcs7Context::encrypt";
+	what = "GMimePkcs7Context::sign (detached)";
 	testsuite_check (what);
 	try {
-		test_encrypt (ctx, FALSE, istream, ostream);
+		test_sign (ctx, TRUE, istream, ostream);
 		testsuite_check_passed ();
 		
-		what = "GMimePkcs7Context::decrypt";
+		what = "GMimePkcs7Context::verify (detached)";
 		testsuite_check (what);
 		g_mime_stream_reset (istream);
 		g_mime_stream_reset (ostream);
-		test_decrypt (ctx, FALSE, istream, ostream);
+		test_verify_detached (ctx, istream, ostream);
 		testsuite_check_passed ();
 	} catch (ex) {
 		testsuite_check_failed ("%s failed: %s", what, ex->message);
@@ -384,17 +411,17 @@ int main (int argc, char **argv)
 	g_mime_stream_reset (istream);
 	ostream = g_mime_stream_mem_new ();
 	
-	what = "GMimePkcs7Context::encrypt+sign";
+	what = "GMimePkcs7Context::encrypt";
 	testsuite_check (what);
 	try {
-		test_encrypt (ctx, TRUE, istream, ostream);
+		test_encrypt (ctx, istream, ostream);
 		testsuite_check_passed ();
 		
-		what = "GMimePkcs7Context::decrypt+verify";
+		what = "GMimePkcs7Context::decrypt";
 		testsuite_check (what);
 		g_mime_stream_reset (istream);
 		g_mime_stream_reset (ostream);
-		test_decrypt (ctx, TRUE, istream, ostream);
+		test_decrypt (ctx, istream, ostream);
 		testsuite_check_passed ();
 	} catch (ex) {
 		testsuite_check_failed ("%s failed: %s", what, ex->message);
@@ -408,7 +435,7 @@ int main (int argc, char **argv)
 	
 	g_mime_shutdown ();
 	
-	if (system ("/bin/rm -rf ./tmp") != 0)
+	if (testsuite_destroy_gpghome () != 0)
 		return EXIT_FAILURE;
 	
 	return testsuite_exit ();
