@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,27 +40,29 @@ extern int verbose;
 
 #define v(x) if (verbose > 3) x
 
+#if 0
 static gboolean
-request_passwd (GMimeCryptoContext *ctx, const char *user_id, const char *prompt_ctx, gboolean reprompt, GMimeStream *response, GError **err)
+request_passwd (GMimeCryptoContext *ctx, const char *user_id, const char *prompt, gboolean reprompt, GMimeStream *response, GError **err)
 {
 	g_mime_stream_write_string (response, "no.secret\n");
 	
 	return TRUE;
 }
+#endif
 
 static GMimeSignatureStatus
 get_sig_status (GMimeSignatureList *signatures)
 {
-	GMimeSignatureStatus status = GMIME_SIGNATURE_STATUS_GOOD;
+	GMimeSignatureStatus status = 0;
 	GMimeSignature *sig;
 	int i;
 	
 	if (!signatures || signatures->array->len == 0)
-		return GMIME_SIGNATURE_STATUS_ERROR;
+		return GMIME_SIGNATURE_STATUS_RED;
 	
 	for (i = 0; i < g_mime_signature_list_length (signatures); i++) {
 		sig = g_mime_signature_list_get_signature (signatures, i);
-		status = MAX (status, sig->status);
+		status |= g_mime_signature_get_status (sig);
 	}
 	
 	return status;
@@ -69,23 +71,21 @@ get_sig_status (GMimeSignatureList *signatures)
 static void
 print_verify_results (GMimeSignatureList *signatures)
 {
+	GMimeSignatureStatus status;
 	GMimeSignature *sig;
 	int count, i;
 	
-	switch (get_sig_status (signatures)) {
-	case GMIME_SIGNATURE_STATUS_GOOD:
-		fputs ("GOOD\n", stdout);
-		break;
-	case GMIME_SIGNATURE_STATUS_BAD:
+	status = get_sig_status (signatures);
+	
+	fputs ("Overall status: ", stdout);
+	if ((status & GMIME_SIGNATURE_STATUS_RED) != 0)
 		fputs ("BAD\n", stdout);
-		break;
-	case GMIME_SIGNATURE_STATUS_ERROR:
-		fputs ("ERROR status\n", stdout);
-		break;
-	default:
-		fputs ("Unknown enum value\n", stdout);
-		break;
-	}
+	else if ((status & GMIME_SIGNATURE_STATUS_GREEN) != 0)
+		fputs ("GOOD\n", stdout);
+	else if ((status & GMIME_SIGNATURE_STATUS_VALID) != 0)
+		fputs ("VALID\n", stdout);
+	else
+		fputs ("UNKNOWN\n", stdout);
 	
 	fputs ("\nSignatures:\n", stdout);
 	count = g_mime_signature_list_length (signatures);
@@ -98,38 +98,35 @@ print_verify_results (GMimeSignatureList *signatures)
 		fprintf (stdout, "\tTrust: ");
 		
 		switch (sig->cert->trust) {
-		case GMIME_CERTIFICATE_TRUST_NONE:
-			fputs ("None\n", stdout);
+		case GMIME_TRUST_UNKNOWN:
+			fputs ("Unknown\n", stdout);
 			break;
-		case GMIME_CERTIFICATE_TRUST_NEVER:
+		case GMIME_TRUST_NEVER:
 			fputs ("Never\n", stdout);
 			break;
-		case GMIME_CERTIFICATE_TRUST_UNDEFINED:
+		case GMIME_TRUST_UNDEFINED:
 			fputs ("Undefined\n", stdout);
 			break;
-		case GMIME_CERTIFICATE_TRUST_MARGINAL:
+		case GMIME_TRUST_MARGINAL:
 			fputs ("Marginal\n", stdout);
 			break;
-		case GMIME_CERTIFICATE_TRUST_FULLY:
-			fputs ("Fully\n", stdout);
+		case GMIME_TRUST_FULL:
+			fputs ("Full\n", stdout);
 			break;
-		case GMIME_CERTIFICATE_TRUST_ULTIMATE:
+		case GMIME_TRUST_ULTIMATE:
 			fputs ("Ultimate\n", stdout);
 			break;
 		}
 		
 		fprintf (stdout, "\tStatus: ");
-		switch (sig->status) {
-		case GMIME_SIGNATURE_STATUS_GOOD:
-			fputs ("GOOD\n", stdout);
-			break;
-		case GMIME_SIGNATURE_STATUS_BAD:
+		if ((sig->status & GMIME_SIGNATURE_STATUS_RED) != 0)
 			fputs ("BAD\n", stdout);
-			break;
-		case GMIME_SIGNATURE_STATUS_ERROR:
-			fputs ("ERROR\n", stdout);
-			break;
-		}
+		else if ((sig->status & GMIME_SIGNATURE_STATUS_GREEN) != 0)
+			fputs ("GOOD\n", stdout);
+		else if ((sig->status & GMIME_SIGNATURE_STATUS_VALID) != 0)
+			fputs ("VALID\n", stdout);
+		else
+			fputs ("UNKNOWN\n", stdout);
 		
 		fprintf (stdout, "\tSignature made on %s", ctime (&sig->created));
 		if (sig->expires != (time_t) 0)
@@ -137,26 +134,84 @@ print_verify_results (GMimeSignatureList *signatures)
 		else
 			fprintf (stdout, "\tSignature never expires\n");
 		
-		if (sig->errors) {
-			fprintf (stdout, "\tErrors: ");
-			if (sig->errors & GMIME_SIGNATURE_ERROR_EXPSIG)
-				fputs ("Expired, ", stdout);
-			if (sig->errors & GMIME_SIGNATURE_ERROR_NO_PUBKEY)
-				fputs ("No Pub Key, ", stdout);
-			if (sig->errors & GMIME_SIGNATURE_ERROR_EXPKEYSIG)
-				fputs ("Key Expired, ", stdout);
-			if (sig->errors & GMIME_SIGNATURE_ERROR_REVKEYSIG)
-				fputs ("Key Revoked", stdout);
-			fputc ('\n', stdout);
-		} else {
-			fprintf (stdout, "\tNo errors for this signer\n");
-		}
+		fprintf (stdout, "\tErrors: ");
+		if (sig->status & GMIME_SIGNATURE_STATUS_KEY_REVOKED)
+			fputs ("Key Revoked, ", stdout);
+		if (sig->status & GMIME_SIGNATURE_STATUS_KEY_EXPIRED)
+			fputs ("Key Expired, ", stdout);
+		if (sig->status & GMIME_SIGNATURE_STATUS_SIG_EXPIRED)
+			fputs ("Sig Expired, ", stdout);
+		if (sig->status & GMIME_SIGNATURE_STATUS_KEY_MISSING)
+			fputs ("Key Missing, ", stdout);
+		if (sig->status & GMIME_SIGNATURE_STATUS_CRL_MISSING)
+			fputs ("CRL Missing, ", stdout);
+		if (sig->status & GMIME_SIGNATURE_STATUS_CRL_TOO_OLD)
+			fputs ("CRL Too Old, ", stdout);
+		if (sig->status & GMIME_SIGNATURE_STATUS_BAD_POLICY)
+			fputs ("Bad Policy, ", stdout);
+		if (sig->status & GMIME_SIGNATURE_STATUS_SYS_ERROR)
+			fputs ("System Error, ", stdout);
+		if (sig->status & GMIME_SIGNATURE_STATUS_TOFU_CONFLICT)
+			fputs ("Tofu Conflict", stdout);
+		if ((sig->status & GMIME_SIGNATURE_STATUS_ERROR_MASK) == 0)
+			fputs ("None", stdout);
+		fputc ('\n', stdout);
 		
 		if (i + 1 < count)
 			fputc ('\n', stdout);
 	}
 }
 
+static GMimeMessage *
+create_message (GMimeObject *body)
+{
+	GMimeFormatOptions *format = g_mime_format_options_get_default ();
+	InternetAddressList *list;
+	InternetAddress *mailbox;
+	GMimeMessage *message;
+	GMimeParser *parser;
+	GMimeStream *stream;
+	
+	message = g_mime_message_new (TRUE);
+	
+	mailbox = internet_address_mailbox_new ("Jeffrey Stedfast", "fejj@helixcode.com");
+	list = g_mime_message_get_from (message);
+	internet_address_list_add (list, mailbox);
+	g_object_unref (mailbox);
+	
+	mailbox = internet_address_mailbox_new ("Jeffrey Stedfast", "fejj@helixcode.com");
+	list = g_mime_message_get_reply_to (message);
+	internet_address_list_add (list, mailbox);
+	g_object_unref (mailbox);
+	
+	mailbox = internet_address_mailbox_new ("Federico Mena-Quintero", "federico@helixcode.com");
+	list = g_mime_message_get_addresses (message, GMIME_ADDRESS_TYPE_TO);
+	internet_address_list_add (list, mailbox);
+	g_object_unref (mailbox);
+	
+	g_mime_message_set_subject (message, "This is a test message", NULL);
+	g_mime_object_set_header ((GMimeObject *) message, "X-Mailer", "main.c", NULL);
+	g_mime_message_set_mime_part (message, body);
+	
+	stream = g_mime_stream_mem_new ();
+	g_mime_object_write_to_stream ((GMimeObject *) message, format, stream);
+	g_mime_stream_reset (stream);
+	g_object_unref (message);
+	
+	/*fprintf (stderr, "-----BEGIN RAW MESSAGE-----\n%.*s-----END RAW MESSAGE-----\n",
+		 GMIME_STREAM_MEM (stream)->buffer->len,
+		 GMIME_STREAM_MEM (stream)->buffer->data);*/
+	
+	parser = g_mime_parser_new ();
+	g_mime_parser_init_with_stream (parser, stream);
+	g_object_unref (stream);
+	
+	message = g_mime_parser_construct_message (parser, NULL);
+	g_object_unref (parser);
+	
+	return message;
+}
+
 #define MULTIPART_SIGNED_CONTENT "This is a test of the emergency broadcast system \
 with an sha1 detach-sign.\n\nFrom now on, there will be text to try and break     \t\
   \nvarious things. For example, the F in \"From\" in the previous line...\n...and \
@@ -169,72 +224,27 @@ test_multipart_signed (GMimeCryptoContext *ctx)
 {
 	GMimeSignatureList *signatures;
 	GMimeMultipartSigned *mps;
-	GMimeDataWrapper *content;
 	GMimeMessage *message;
-	GMimeStream *stream;
-	GMimeParser *parser;
+	GMimeTextPart *part;
 	GError *err = NULL;
-	GMimePart *part;
 	Exception *ex;
 	
-	part = g_mime_part_new_with_type ("text", "plain");
-	
-	stream = g_mime_stream_mem_new ();
-	g_mime_stream_write_string (stream, MULTIPART_SIGNED_CONTENT);
-#if 0
-	"This is a test of the emergency broadcast system with an sha1 detach-sign.\n\n"
-		"From now on, there will be text to try and break     \t  \n"
-		"various things. For example, the F in \"From\" in the previous line...\n"
-		"...and the first dot of this line have been pre-encoded in the QP encoding "
-		"in order to test that GMime properly treats MIME part content as opaque.\n"
-		"If this still verifies okay, then we have ourselves a winner I guess...\n";
-#endif
-	
-	g_mime_stream_reset (stream);
-	content = g_mime_data_wrapper_new_with_stream (stream, GMIME_CONTENT_ENCODING_DEFAULT);
-	g_object_unref (stream);
-	
-	g_mime_part_set_content_object (part, content);
-	g_object_unref (content);
-	
-	/* create the multipart/signed container part */
-	mps = g_mime_multipart_signed_new ();
+	part = g_mime_text_part_new_with_subtype ("plain");
+	g_mime_text_part_set_text (part, MULTIPART_SIGNED_CONTENT);
 	
 	/* sign the part */
-	g_mime_multipart_signed_sign (mps, GMIME_OBJECT (part), ctx, "alice@example.net",
-				      GMIME_DIGEST_ALGO_SHA1, &err);
+	mps = g_mime_multipart_signed_sign (ctx, (GMimeObject *) part, "mimekit@example.com", &err);
 	g_object_unref (part);
 	
 	if (err != NULL) {
 		ex = exception_new ("signing failed: %s", err->message);
-		g_object_unref (mps);
 		g_error_free (err);
 		throw (ex);
 	}
 	
-	message = g_mime_message_new (TRUE);
-	g_mime_message_set_sender (message, "\"Jeffrey Stedfast\" <fejj@helixcode.com>");
-	g_mime_message_set_reply_to (message, "fejj@helixcode.com");
-	g_mime_message_add_recipient (message, GMIME_RECIPIENT_TYPE_TO,
-				      "Federico Mena-Quintero",
-				      "federico@helixcode.com");
-	g_mime_message_set_subject (message, "This is a test message");
-	g_mime_object_set_header ((GMimeObject *) message, "X-Mailer", "main.c");
-	g_mime_message_set_mime_part (message, GMIME_OBJECT (mps));
+	message = create_message ((GMimeObject *) mps);
 	g_object_unref (mps);
 	
-	stream = g_mime_stream_mem_new ();
-	g_mime_object_write_to_stream ((GMimeObject *) message, stream);
-	g_mime_stream_reset (stream);
-	g_object_unref (message);
-	
-	parser = g_mime_parser_new ();
-	g_mime_parser_init_with_stream (parser, stream);
-	g_object_unref (stream);
-	
-	message = g_mime_parser_construct_message (parser);
-	g_object_unref (parser);
-	
 	if (!GMIME_IS_MULTIPART_SIGNED (message->mime_part)) {
 		ex = exception_new ("resultant top-level mime part not a multipart/signed?");
 		g_object_unref (message);
@@ -243,10 +253,8 @@ test_multipart_signed (GMimeCryptoContext *ctx)
 	
 	mps = (GMimeMultipartSigned *) message->mime_part;
 	
-	v(fputs ("Trying to verify signature... ", stdout));
-	if (!(signatures = g_mime_multipart_signed_verify (mps, ctx, &err))) {
+	if (!(signatures = g_mime_multipart_signed_verify (mps, 0, &err))) {
 		ex = exception_new ("%s", err->message);
-		v(fputs ("failed.\n", stdout));
 		g_error_free (err);
 		throw (ex);
 	}
@@ -257,133 +265,147 @@ test_multipart_signed (GMimeCryptoContext *ctx)
 	g_object_unref (message);
 }
 
-#define MULTIPART_ENCRYPTED_CONTENT "This is a test of multipart/encrypted.\n"
+#define SIGNED_CONTENT "This is a test of application/pkcs7-mime; smime-type=signed-data.\n"
 
 static void
-test_multipart_encrypted (GMimeCryptoContext *ctx, gboolean sign)
+test_pkcs7_mime_sign (void)
 {
-	GMimeStream *cleartext, *stream;
-	GMimeMultipartEncrypted *mpe;
-	GMimeDecryptResult *result;
-	GMimeDataWrapper *content;
-	GMimeObject *decrypted;
-	GPtrArray *recipients;
+	GMimeApplicationPkcs7Mime *pkcs7_mime;
+	GMimeSignatureList *signatures;
 	GMimeMessage *message;
-	Exception *ex = NULL;
-	GMimeParser *parser;
-	GByteArray *buf[2];
+	GMimeObject *entity;
+	GMimeTextPart *part;
 	GError *err = NULL;
-	GMimePart *part;
+	Exception *ex;
+	char *text;
 	
-	cleartext = g_mime_stream_mem_new ();
-	g_mime_stream_write_string (cleartext, MULTIPART_ENCRYPTED_CONTENT);
-	g_mime_stream_reset (cleartext);
+	part = g_mime_text_part_new ();
+	g_mime_text_part_set_text (part, SIGNED_CONTENT);
 	
-	content = g_mime_data_wrapper_new ();
-	g_mime_data_wrapper_set_stream (content, cleartext);
-	g_object_unref (cleartext);
-	
-	part = g_mime_part_new_with_type ("text", "plain");
-	g_mime_part_set_content_object (part, content);
-	g_object_unref (content);
-	
-	/* hold onto this for comparison later */
-	cleartext = g_mime_stream_mem_new ();
-	g_mime_object_write_to_stream ((GMimeObject *) part, cleartext);
-	g_mime_stream_reset (cleartext);
-	
-	/* create the multipart/encrypted container part */
-	mpe = g_mime_multipart_encrypted_new ();
-	
-	/* encrypt the part */
-	recipients = g_ptr_array_new ();
-	g_ptr_array_add (recipients, "alice@example.net");
-	g_mime_multipart_encrypted_encrypt (mpe, GMIME_OBJECT (part), ctx, sign,
-					    "alice@example.net", GMIME_DIGEST_ALGO_SHA256,
-					    recipients, &err);
-	g_ptr_array_free (recipients, TRUE);
+	pkcs7_mime = g_mime_application_pkcs7_mime_sign ((GMimeObject *) part, "mimekit@example.com", &err);
 	g_object_unref (part);
 	
 	if (err != NULL) {
-		ex = exception_new ("encryption failed: %s", err->message);
-		g_object_unref (cleartext);
-		g_object_unref (mpe);
+		ex = exception_new ("sign failed: %s", err->message);
 		g_error_free (err);
 		throw (ex);
 	}
 	
-	message = g_mime_message_new (TRUE);
-	g_mime_message_set_sender (message, "\"Jeffrey Stedfast\" <fejj@helixcode.com>");
-	g_mime_message_set_reply_to (message, "fejj@helixcode.com");
-	g_mime_message_add_recipient (message, GMIME_RECIPIENT_TYPE_TO,
-				      "Federico Mena-Quintero",
-				      "federico@helixcode.com");
-	g_mime_message_set_subject (message, "This is a test message");
-	g_mime_object_set_header ((GMimeObject *) message, "X-Mailer", "main.c");
-	g_mime_message_set_mime_part (message, GMIME_OBJECT (mpe));
-	g_object_unref (mpe);
+	message = create_message ((GMimeObject *) pkcs7_mime);
+	g_object_unref (pkcs7_mime);
 	
-	stream = g_mime_stream_mem_new ();
-	g_mime_object_write_to_stream ((GMimeObject *) message, stream);
-	g_mime_stream_reset (stream);
-	g_object_unref (message);
-	
-	parser = g_mime_parser_new ();
-	g_mime_parser_init_with_stream (parser, stream);
-	g_object_unref (stream);
-	
-	message = g_mime_parser_construct_message (parser);
-	g_object_unref (parser);
-	
-	if (!GMIME_IS_MULTIPART_ENCRYPTED (message->mime_part)) {
-		ex = exception_new ("resultant top-level mime part not a multipart/encrypted?");
+	if (!GMIME_IS_APPLICATION_PKCS7_MIME (message->mime_part)) {
+		ex = exception_new ("resultant top-level mime part not an application/pkcs7-mime?");
 		g_object_unref (message);
 		throw (ex);
 	}
 	
-	mpe = (GMimeMultipartEncrypted *) message->mime_part;
+	pkcs7_mime = (GMimeApplicationPkcs7Mime *) message->mime_part;
 	
-	/* okay, now to test our decrypt function... */
-	decrypted = g_mime_multipart_encrypted_decrypt (mpe, ctx, &result, &err);
-	if (!decrypted || err != NULL) {
-		ex = exception_new ("decryption failed: %s", err->message);
-		g_object_unref (cleartext);
+	if (!(signatures = g_mime_application_pkcs7_mime_verify (pkcs7_mime, 0, &entity, &err))) {
+		ex = exception_new ("verify failed: %s", err->message);
+		g_object_unref (message);
 		g_error_free (err);
 		throw (ex);
 	}
 	
-	if (result->signatures)
-		v(print_verify_results (result->signatures));
+	v(print_verify_results (signatures));
+	g_object_unref (signatures);
+	g_object_unref (message);
 	
-	if (sign) {
-		if (!result->signatures || get_sig_status (result->signatures) != GMIME_SIGNATURE_STATUS_GOOD)
-			ex = exception_new ("signature status expected to be GOOD");
-	} else {
-		if (result->signatures)
-			ex = exception_new ("signature status expected to be NONE");
+	/* TODO: verify extracted content... */
+	if (!GMIME_IS_TEXT_PART (entity)) {
+		g_object_unref (entity);
+		throw (exception_new ("extracted entity was not a text/plain part?"));
+	}
+	
+	text = g_mime_text_part_get_text ((GMimeTextPart *) entity);
+	if (strcmp (SIGNED_CONTENT, text) != 0) {
+		ex = exception_new ("text part content does not match");
+		g_object_unref (entity);
+		g_free (text);
+		throw (ex);
+	}
+	
+	g_object_unref (entity);
+	g_free (text);
+}
+
+#define ENCRYPTED_CONTENT "This is a test of application/pkcs7-mime; smime-type=enveloped-data.\n"
+
+static void
+test_pkcs7_mime_encrypt (void)
+{
+	GMimeApplicationPkcs7Mime *pkcs7_mime;
+	GMimeDecryptResult *result;
+	GPtrArray *recipients;
+	GMimeMessage *message;
+	GMimeObject *entity;
+	GMimeTextPart *part;
+	GError *err = NULL;
+	Exception *ex;
+	char *text;
+	
+	part = g_mime_text_part_new ();
+	g_mime_text_part_set_text (part, ENCRYPTED_CONTENT);
+	
+	recipients = g_ptr_array_new ();
+	g_ptr_array_add (recipients, "mimekit@example.com");
+	
+	pkcs7_mime = g_mime_application_pkcs7_mime_encrypt ((GMimeObject *) part, GMIME_ENCRYPT_ALWAYS_TRUST, recipients, &err);
+	g_ptr_array_free (recipients, TRUE);
+	g_object_unref (part);
+	
+	if (err != NULL) {
+		ex = exception_new ("encrypt failed: %s", err->message);
+		g_error_free (err);
+		throw (ex);
+	}
+	
+	message = create_message ((GMimeObject *) pkcs7_mime);
+	g_object_unref (pkcs7_mime);
+	
+	if (!GMIME_IS_APPLICATION_PKCS7_MIME (message->mime_part)) {
+		ex = exception_new ("resultant top-level mime part not an application/pkcs7-mime?");
+		g_object_unref (message);
+		throw (ex);
+	}
+	
+	pkcs7_mime = (GMimeApplicationPkcs7Mime *) message->mime_part;
+	
+	if (!(entity = g_mime_application_pkcs7_mime_decrypt (pkcs7_mime, 0, NULL, &result, &err))) {
+		ex = exception_new ("decrypt failed: %s", err->message);
+		g_object_unref (message);
+		g_error_free (err);
+		throw (ex);
+	}
+	
+	g_object_unref (message);
+	
+	if (result->signatures) {
+		g_object_unref (result);
+		throw (exception_new ("signature status expected to be NONE"));
 	}
 	
 	g_object_unref (result);
 	
-	if (ex != NULL) {
-		g_object_unref (cleartext);
+	/* TODO: verify decrypted content... */
+	if (!GMIME_IS_TEXT_PART (entity)) {
+		g_object_unref (entity);
+		throw (exception_new ("decrypted entity was not a text/plain part?"));
+	}
+	
+	text = g_mime_text_part_get_text ((GMimeTextPart *) entity);
+	
+	if (strcmp (ENCRYPTED_CONTENT, text) != 0) {
+		ex = exception_new ("text part content does not match");
+		g_object_unref (entity);
+		g_free (text);
 		throw (ex);
 	}
 	
-	stream = g_mime_stream_mem_new ();
-	g_mime_object_write_to_stream (decrypted, stream);
-	
-	buf[0] = GMIME_STREAM_MEM (cleartext)->buffer;
-	buf[1] = GMIME_STREAM_MEM (stream)->buffer;
-	
-	if (buf[0]->len != buf[1]->len || memcmp (buf[0]->data, buf[1]->data, buf[0]->len) != 0)
-		ex = exception_new ("decrypted data does not match original cleartext");
-	
-	g_object_unref (cleartext);
-	g_object_unref (stream);
-	
-	if (ex != NULL)
-		throw (ex);
+	g_object_unref (entity);
+	g_free (text);
 }
 
 static void
@@ -416,17 +438,11 @@ int main (int argc, char *argv[])
 	char *key;
 	int i;
 	
-	g_mime_init (0);
+	g_mime_init ();
 	
 	testsuite_init (argc, argv);
 	
-	/* reset .gnupg config directory */
-	if (system ("/bin/rm -rf ./tmp") != 0)
-		return EXIT_FAILURE;
-	if (system ("/bin/mkdir ./tmp") != 0)
-		return EXIT_FAILURE;
-	g_setenv ("GNUPGHOME", "./tmp/.gnupg", 1);
-	if (system ("/usr/bin/gpg --list-keys > /dev/null 2>&1") != 0)
+	if (testsuite_setup_gpghome ("gpgsm") != 0)
 		return EXIT_FAILURE;
 	
 	for (i = 1; i < argc; i++) {
@@ -441,16 +457,16 @@ int main (int argc, char *argv[])
 	
 	testsuite_start ("S/MIME implementation");
 	
-	ctx = g_mime_pkcs7_context_new (request_passwd);
-	g_mime_pkcs7_context_set_always_trust ((GMimePkcs7Context *) ctx, TRUE);
+	ctx = g_mime_pkcs7_context_new ();
+	//g_mime_crypto_context_set_request_password (ctx, request_passwd);
 	
 	testsuite_check ("GMimePkcs7Context::import");
 	try {
-		key = g_build_filename (datadir, "gmime-cert.p7", NULL);
+		key = g_build_filename (datadir, "certificate-authority.crt", NULL);
 		import_key (ctx, key);
 		g_free (key);
 		
-		key = g_build_filename (datadir, "gmime-cert.p12", NULL);
+		key = g_build_filename (datadir, "smime.p12", NULL);
 		import_key (ctx, key);
 		g_free (key);
 		
@@ -468,29 +484,29 @@ int main (int argc, char *argv[])
 		testsuite_check_failed ("multipart/signed failed: %s", ex->message);
 	} finally;
 	
-	testsuite_check ("multipart/encrypted");
+	testsuite_check ("application/pkcs7-mime; smime-type=signed-data");
 	try {
-		test_multipart_encrypted (ctx, FALSE);
+		test_pkcs7_mime_sign ();
+		testsuite_check_passed ();
+	} catch (ex) {
+		testsuite_check_failed ("multipart/signed failed: %s", ex->message);
+	} finally;
+	
+	testsuite_check ("application/pkcs7-mime; smime-type=enveloped-data");
+	try {
+		test_pkcs7_mime_encrypt ();
 		testsuite_check_passed ();
 	} catch (ex) {
 		testsuite_check_failed ("multipart/encrypted failed: %s", ex->message);
 	} finally;
 	
-	testsuite_check ("multipart/encrypted+sign");
-	try {
-		test_multipart_encrypted (ctx, TRUE);
-		testsuite_check_passed ();
-	} catch (ex) {
-		testsuite_check_failed ("multipart/encrypted+sign failed: %s", ex->message);
-	} finally;
-	
 	g_object_unref (ctx);
 	
 	testsuite_end ();
 	
 	g_mime_shutdown ();
 	
-	if (system ("/bin/rm -rf ./tmp") != 0)
+	if (testsuite_destroy_gpghome () != 0)
 		return EXIT_FAILURE;
 	
 	return testsuite_exit ();
